@@ -12,6 +12,14 @@
 
 G_BEGIN_DECLS
 
+/* enumerations from "secret-collection.h" */
+GType secret_collection_flags_get_type (void) G_GNUC_CONST;
+#define SECRET_TYPE_COLLECTION_FLAGS (secret_collection_flags_get_type ())
+
+/* enumerations from "secret-item.h" */
+GType secret_item_flags_get_type (void) G_GNUC_CONST;
+#define SECRET_TYPE_ITEM_FLAGS (secret_item_flags_get_type ())
+
 /* enumerations from "secret-schema.h" */
 GType secret_schema_attribute_type_get_type (void) G_GNUC_CONST;
 #define SECRET_TYPE_SCHEMA_ATTRIBUTE_TYPE (secret_schema_attribute_type_get_type ())
@@ -25,6 +33,8 @@ GType secret_service_flags_get_type (void) G_GNUC_CONST;
 /* enumerations from "secret-types.h" */
 GType secret_error_get_type (void) G_GNUC_CONST;
 #define SECRET_TYPE_ERROR (secret_error_get_type ())
+GType secret_search_flags_get_type (void) G_GNUC_CONST;
+#define SECRET_TYPE_SEARCH_FLAGS (secret_search_flags_get_type ())
 G_END_DECLS
 
 #endif /* __SECRET_ENUM_TYPES_H__ */
