@@ -16,7 +16,9 @@
 
 #include "secret-collection.h"
 #include "secret-dbus-generated.h"
+#include "secret-enum-types.h"
 #include "secret-item.h"
+#include "secret-paths.h"
 #include "secret-private.h"
 #include "secret-service.h"
 #include "secret-types.h"
@@ -64,9 +66,18 @@
  * The class for #SecretItem.
  */
 
+/**
+ * SecretItemFlags:
+ * @SECRET_ITEM_NONE: no flags
+ * @SECRET_ITEM_LOAD_SECRET: a secret has been (or should be) loaded for #SecretItem
+ *
+ * Flags which determine which parts of the #SecretItem proxy are initialized.
+ */
+
 enum {
 	PROP_0,
 	PROP_SERVICE,
+	PROP_FLAGS,
 	PROP_ATTRIBUTES,
 	PROP_LABEL,
 	PROP_LOCKED,
@@ -74,10 +85,15 @@ enum {
 	PROP_MODIFIED
 };
 
-/* Thread safe: no changes between construct and finalize */
 struct _SecretItemPrivate {
+	/* No changes between construct and finalize */
 	SecretService *service;
+	SecretItemFlags init_flags;
 	GCancellable *cancellable;
+
+	/* Locked by mutex */
+	GMutex mutex;
+	SecretValue *value;
 };
 
 static GInitableIface *secret_item_initable_parent_iface = NULL;
@@ -98,6 +114,7 @@ secret_item_init (SecretItem *self)
 {
 	self->pv = G_TYPE_INSTANCE_GET_PRIVATE (self, SECRET_TYPE_ITEM, SecretItemPrivate);
 	self->pv->cancellable = g_cancellable_new ();
+	g_mutex_init (&self->pv->mutex);
 }
 
 static void
@@ -134,6 +151,24 @@ on_set_label (GObject *source,
 	g_object_unref (self);
 }
 
+
+static void
+item_take_service (SecretItem *self,
+                   SecretService *service)
+{
+	if (service == NULL)
+		return;
+
+	g_return_if_fail (self->pv->service == NULL);
+
+	self->pv->service = service;
+	g_object_add_weak_pointer (G_OBJECT (self->pv->service),
+	                           (gpointer *)&self->pv->service);
+
+	/* Yes, we expect that the service will stay around */
+	g_object_unref (service);
+}
+
 static void
 secret_item_set_property (GObject *obj,
                           guint prop_id,
@@ -144,11 +179,10 @@ secret_item_set_property (GObject *obj,
 
 	switch (prop_id) {
 	case PROP_SERVICE:
-		g_return_if_fail (self->pv->service == NULL);
-		self->pv->service = g_value_get_object (value);
-		if (self->pv->service)
-			g_object_add_weak_pointer (G_OBJECT (self->pv->service),
-			                           (gpointer *)&self->pv->service);
+		item_take_service (self, g_value_dup_object (value));
+		break;
+	case PROP_FLAGS:
+		self->pv->init_flags = g_value_get_flags (value);
 		break;
 	case PROP_ATTRIBUTES:
 		secret_item_set_attributes (self, g_value_get_boxed (value),
@@ -178,6 +212,9 @@ secret_item_get_property (GObject *obj,
 	case PROP_SERVICE:
 		g_value_set_object (value, self->pv->service);
 		break;
+	case PROP_FLAGS:
+		g_value_set_flags (value, secret_item_get_flags (self));
+		break;
 	case PROP_ATTRIBUTES:
 		g_value_take_boxed (value, secret_item_get_attributes (self));
 		break;
@@ -219,6 +256,7 @@ secret_item_finalize (GObject *obj)
 		                              (gpointer *)&self->pv->service);
 
 	g_object_unref (self->pv->cancellable);
+	g_mutex_clear (&self->pv->mutex);
 
 	G_OBJECT_CLASS (secret_item_parent_class)->finalize (obj);
 }
@@ -285,6 +323,17 @@ secret_item_class_init (SecretItemClass *klass)
 	            g_param_spec_object ("service", "Service", "Secret Service",
 	                                 SECRET_TYPE_SERVICE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 
+	/**
+	 * SecretService:flags:
+	 *
+	 * A set of flags describing which parts of the secret item have
+	 * been initialized.
+	 */
+	g_object_class_install_property (gobject_class, PROP_FLAGS,
+	             g_param_spec_flags ("flags", "Flags", "Item flags",
+	                                 secret_item_flags_get_type (), SECRET_ITEM_NONE,
+	                                 G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
+
 	/**
 	 * SecretItem:attributes:
 	 *
@@ -344,11 +393,71 @@ secret_item_class_init (SecretItemClass *klass)
 	g_type_class_add_private (gobject_class, sizeof (SecretItemPrivate));
 }
 
+typedef struct {
+	GCancellable *cancellable;
+} InitClosure;
+
+static void
+init_closure_free (gpointer data)
+{
+	InitClosure *closure = data;
+	g_clear_object (&closure->cancellable);
+	g_slice_free (InitClosure, closure);
+}
+
+static gboolean
+item_ensure_for_flags_sync (SecretItem *self,
+                            SecretItemFlags flags,
+                            GCancellable *cancellable,
+                            GError **error)
+{
+	if (flags & SECRET_ITEM_LOAD_SECRET && !secret_item_get_locked (self)) {
+		if (!secret_item_load_secret_sync (self, cancellable, error))
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
+
+static void
+on_init_load_secret (GObject *source,
+                     GAsyncResult *result,
+                     gpointer user_data)
+{
+	GSimpleAsyncResult *async = G_SIMPLE_ASYNC_RESULT (user_data);
+	SecretItem *self = SECRET_ITEM (source);
+	GError *error = NULL;
+
+	if (!secret_item_load_secret_finish (self, result, &error))
+		g_simple_async_result_take_error (async, error);
+
+	g_simple_async_result_complete (async);
+	g_object_unref (async);
+}
+
+static void
+item_ensure_for_flags_async (SecretItem *self,
+                             SecretItemFlags flags,
+                             GSimpleAsyncResult *async)
+{
+	InitClosure *init = g_simple_async_result_get_op_res_gpointer (async);
+
+	if (flags & SECRET_ITEM_LOAD_SECRET && !secret_item_get_locked (self))
+		secret_item_load_secret (self, init->cancellable,
+		                         on_init_load_secret, g_object_ref (async));
+
+	else
+		g_simple_async_result_complete (async);
+}
+
 static gboolean
 secret_item_initable_init (GInitable *initable,
                            GCancellable *cancellable,
                            GError **error)
 {
+	SecretItem *self;
+	SecretService *service;
 	GDBusProxy *proxy;
 
 	if (!secret_item_initable_parent_iface->init (initable, cancellable, error))
@@ -363,7 +472,16 @@ secret_item_initable_init (GInitable *initable,
 		return FALSE;
 	}
 
-	return TRUE;
+	self = SECRET_ITEM (initable);
+	if (!self->pv->service) {
+		service = secret_service_get_sync (SECRET_SERVICE_NONE, cancellable, error);
+		if (service == NULL)
+			return FALSE;
+		else
+			item_take_service (self, service);
+	}
+
+	return item_ensure_for_flags_sync (self, self->pv->init_flags, cancellable, error);
 }
 
 static void
@@ -374,12 +492,37 @@ secret_item_initable_iface (GInitableIface *iface)
 	iface->init = secret_item_initable_init;
 }
 
+static void
+on_init_service (GObject *source,
+                 GAsyncResult *result,
+                 gpointer user_data)
+{
+	GSimpleAsyncResult *async = G_SIMPLE_ASYNC_RESULT (user_data);
+	SecretItem *self = SECRET_ITEM (g_async_result_get_source_object (user_data));
+	SecretService *service;
+	GError *error = NULL;
+
+	service = secret_service_get_finish (result, &error);
+	if (error == NULL) {
+		item_take_service (self, service);
+		item_ensure_for_flags_async (self, self->pv->init_flags, async);
+
+	} else {
+		g_simple_async_result_take_error (async, error);
+		g_simple_async_result_complete (async);
+	}
+
+	g_object_unref (self);
+	g_object_unref (async);
+}
+
 static void
 on_init_base (GObject *source,
               GAsyncResult *result,
               gpointer user_data)
 {
 	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
+	InitClosure *init = g_simple_async_result_get_op_res_gpointer (res);
 	SecretItem *self = SECRET_ITEM (source);
 	GDBusProxy *proxy = G_DBUS_PROXY (self);
 	GError *error = NULL;
@@ -387,14 +530,22 @@ on_init_base (GObject *source,
 	if (!secret_item_async_initable_parent_iface->init_finish (G_ASYNC_INITABLE (self),
 	                                                           result, &error)) {
 		g_simple_async_result_take_error (res, error);
+		g_simple_async_result_complete (res);
 
 	} else if (!_secret_util_have_cached_properties (proxy)) {
 		g_simple_async_result_set_error (res, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD,
 		                                 "No such secret item at path: %s",
 		                                 g_dbus_proxy_get_object_path (proxy));
+		g_simple_async_result_complete (res);
+
+	} else if (self->pv->service == NULL) {
+		secret_service_get (SECRET_SERVICE_NONE, init->cancellable,
+		                    on_init_service, g_object_ref (res));
+
+	} else {
+		item_ensure_for_flags_async (self, self->pv->init_flags, res);
 	}
 
-	g_simple_async_result_complete (res);
 	g_object_unref (res);
 }
 
@@ -406,9 +557,13 @@ secret_item_async_initable_init_async (GAsyncInitable *initable,
                                        gpointer user_data)
 {
 	GSimpleAsyncResult *res;
+	InitClosure *init;
 
 	res = g_simple_async_result_new (G_OBJECT (initable), callback, user_data,
 	                                 secret_item_async_initable_init_async);
+	init = g_slice_new0 (InitClosure);
+	init->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
+	g_simple_async_result_set_op_res_gpointer (res, init, init_closure_free);
 
 	secret_item_async_initable_parent_iface->init_async (initable, io_priority,
 	                                                     cancellable,
@@ -441,116 +596,6 @@ secret_item_async_initable_iface (GAsyncInitableIface *iface)
 	iface->init_finish = secret_item_async_initable_init_finish;
 }
 
-/**
- * secret_item_new:
- * @service: a secret service object
- * @item_path: the D-Bus path of the collection
- * @cancellable: optional cancellation object
- * @callback: called when the operation completes
- * @user_data: data to be passed to the callback
- *
- * Get a new item proxy for a secret item in the secret service.
- *
- * This method will return immediately and complete asynchronously.
- */
-void
-secret_item_new (SecretService *service,
-                 const gchar *item_path,
-                 GCancellable *cancellable,
-                 GAsyncReadyCallback callback,
-                 gpointer user_data)
-{
-	GDBusProxy *proxy;
-
-	g_return_if_fail (SECRET_IS_SERVICE (service));
-	g_return_if_fail (item_path != NULL);
-	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
-
-	proxy = G_DBUS_PROXY (service);
-
-	g_async_initable_new_async (SECRET_SERVICE_GET_CLASS (service)->item_gtype,
-	                            G_PRIORITY_DEFAULT, cancellable, callback, user_data,
-	                            "g-flags", G_DBUS_CALL_FLAGS_NONE,
-	                            "g-interface-info", _secret_gen_item_interface_info (),
-	                            "g-name", g_dbus_proxy_get_name (proxy),
-	                            "g-connection", g_dbus_proxy_get_connection (proxy),
-	                            "g-object-path", item_path,
-	                            "g-interface-name", SECRET_ITEM_INTERFACE,
-	                            "service", service,
-	                            NULL);
-}
-
-/**
- * secret_item_new_finish:
- * @result: the asynchronous result passed to the callback
- * @error: location to place an error on failure
- *
- * Finish asynchronous operation to get a new item proxy for an secret
- * item in the secret service.
- *
- * Returns: (transfer full): the new item, which should be unreferenced
- *          with g_object_unref()
- */
-SecretItem *
-secret_item_new_finish (GAsyncResult *result,
-                        GError **error)
-{
-	GObject *object;
-	GObject *source_object;
-
-	source_object = g_async_result_get_source_object (result);
-	object = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object),
-	                                      result, error);
-	g_object_unref (source_object);
-
-	if (object == NULL)
-		return NULL;
-
-	return SECRET_ITEM (object);
-}
-
-/**
- * secret_item_new_sync:
- * @service: a secret service object
- * @item_path: the D-Bus path of the item
- * @cancellable: optional cancellation object
- * @error: location to place an error on failure
- *
- * Get a new item proxy for a secret item in the secret service.
- *
- * This method may block indefinitely and should not be used in user interface
- * threads.
- *
- * Returns: (transfer full): the new item, which should be unreferenced
- *          with g_object_unref()
- */
-SecretItem *
-secret_item_new_sync (SecretService *service,
-                      const gchar *item_path,
-                      GCancellable *cancellable,
-                      GError **error)
-{
-	GDBusProxy *proxy;
-
-	g_return_val_if_fail (SECRET_IS_SERVICE (service), NULL);
-	g_return_val_if_fail (item_path != NULL, NULL);
-	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
-	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
-
-	proxy = G_DBUS_PROXY (service);
-
-	return g_initable_new (SECRET_SERVICE_GET_CLASS (service)->item_gtype,
-	                       cancellable, error,
-	                       "g-flags", G_DBUS_CALL_FLAGS_NONE,
-	                       "g-interface-info", _secret_gen_item_interface_info (),
-	                       "g-name", g_dbus_proxy_get_name (proxy),
-	                       "g-connection", g_dbus_proxy_get_connection (proxy),
-	                       "g-object-path", item_path,
-	                       "g-interface-name", SECRET_ITEM_INTERFACE,
-	                       "service", service,
-	                       NULL);
-}
-
 /**
  * secret_item_refresh:
  * @self: the collection
@@ -571,10 +616,41 @@ secret_item_refresh (SecretItem *self)
 	                             NULL, NULL, NULL);
 }
 
+void
+_secret_item_set_cached_secret (SecretItem *self,
+                                SecretValue *value)
+{
+	SecretValue *other = NULL;
+	gboolean updated = FALSE;
+
+	g_return_if_fail (SECRET_IS_ITEM (self));
+
+	if (value != NULL)
+		secret_value_ref (value);
+
+	g_mutex_lock (&self->pv->mutex);
+
+	if (value != self->pv->value) {
+		other = self->pv->value;
+		self->pv->value = value;
+		updated = TRUE;
+	} else {
+		other = value;
+	}
+
+	g_mutex_unlock (&self->pv->mutex);
+
+	if (other != NULL)
+		secret_value_unref (other);
+
+	if (updated)
+		g_object_notify (G_OBJECT (self), "flags");
+}
 
 typedef struct {
 	GCancellable *cancellable;
 	SecretItem *item;
+	SecretValue *value;
 } CreateClosure;
 
 static void
@@ -583,6 +659,7 @@ create_closure_free (gpointer data)
 	CreateClosure *closure = data;
 	g_clear_object (&closure->cancellable);
 	g_clear_object (&closure->item);
+	secret_value_unref (closure->value);
 	g_slice_free (CreateClosure, closure);
 }
 
@@ -595,10 +672,13 @@ on_create_item (GObject *source,
 	CreateClosure *closure = g_simple_async_result_get_op_res_gpointer (res);
 	GError *error = NULL;
 
-	closure->item = secret_item_new_finish (result, &error);
+	closure->item = secret_item_new_for_dbus_path_finish (result, &error);
 	if (error != NULL)
 		g_simple_async_result_take_error (res, error);
 
+	/* As a convenince mark down the SecretValue on the item */
+	_secret_item_set_cached_secret (closure->item, closure->value);
+
 	g_simple_async_result_complete (res);
 	g_object_unref (res);
 }
@@ -614,10 +694,11 @@ on_create_path (GObject *source,
 	GError *error = NULL;
 	gchar *path;
 
-	path = secret_service_create_item_path_finish (service, result, &error);
+	path = secret_service_create_item_dbus_path_finish (service, result, &error);
 	if (error == NULL) {
-		secret_item_new (service, path, closure->cancellable,
-		                 on_create_item, g_object_ref (res));
+		secret_item_new_for_dbus_path (service, path, SECRET_ITEM_NONE,
+		                               closure->cancellable, on_create_item,
+		                               g_object_ref (res));
 	} else {
 		g_simple_async_result_take_error (res, error);
 		g_simple_async_result_complete (res);
@@ -641,7 +722,7 @@ item_properties_new (const gchar *label,
 	                     SECRET_ITEM_INTERFACE ".Label",
 	                     g_variant_ref_sink (value));
 
-	value = _secret_util_variant_for_attributes (attributes, NULL);
+	value = _secret_attributes_to_variant (attributes, NULL);
 	g_hash_table_insert (properties,
 	                     SECRET_ITEM_INTERFACE ".Attributes",
 	                     g_variant_ref_sink (value));
@@ -696,6 +777,7 @@ secret_item_create (SecretCollection *collection,
 	                                 secret_item_create);
 	closure = g_slice_new0 (CreateClosure);
 	closure->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
+	closure->value = secret_value_ref (value);
 	g_simple_async_result_set_op_res_gpointer (res, closure, create_closure_free);
 
 	properties = item_properties_new (label, attributes);
@@ -703,9 +785,9 @@ secret_item_create (SecretCollection *collection,
 
 	collection_path = g_dbus_proxy_get_object_path (G_DBUS_PROXY (collection));
 
-	secret_service_create_item_path (service, collection_path, properties,
-	                                 value, replace, cancellable,
-	                                 on_create_path, g_object_ref (res));
+	secret_service_create_item_dbus_path (service, collection_path, properties,
+	                                      value, replace, cancellable,
+	                                      on_create_path, g_object_ref (res));
 
 	g_hash_table_unref (properties);
 	g_object_unref (service);
@@ -795,11 +877,12 @@ secret_item_create_sync (SecretCollection *collection,
 
 	collection_path = g_dbus_proxy_get_object_path (G_DBUS_PROXY (collection));
 
-	path = secret_service_create_item_path_sync (service, collection_path, properties,
-	                                             value, replace, cancellable, error);
+	path = secret_service_create_item_dbus_path_sync (service, collection_path, properties,
+	                                                  value, replace, cancellable, error);
 
 	if (path != NULL) {
-		item = secret_item_new_sync (service, path, cancellable, error);
+		item = secret_item_new_for_dbus_path_sync (service, path, SECRET_ITEM_NONE,
+		                                           cancellable, error);
 		g_free (path);
 	}
 
@@ -818,10 +901,8 @@ on_item_deleted (GObject *source,
 	SecretItem *self = SECRET_ITEM (g_async_result_get_source_object (user_data));
 	GError *error = NULL;
 
-	if (secret_service_delete_path_finish (SECRET_SERVICE (source), result, &error)) {
+	if (_secret_service_delete_path_finish (SECRET_SERVICE (source), result, &error))
 		g_simple_async_result_set_op_res_gboolean (res, TRUE);
-		g_object_run_dispose (G_OBJECT (self));
-	}
 
 	if (error != NULL)
 		g_simple_async_result_take_error (res, error);
@@ -937,6 +1018,36 @@ secret_item_delete_sync (SecretItem *self,
 	return ret;
 }
 
+/**
+ * secret_item_get_flags:
+ * @self: the secret item proxy
+ *
+ * Get the flags representing what features of the #SecretItem proxy
+ * have been initialized.
+ *
+ * Use secret_item_load_secret() to initialize further features
+ * and change the flags.
+ *
+ * Returns: the flags for features initialized
+ */
+SecretItemFlags
+secret_item_get_flags (SecretItem *self)
+{
+	SecretServiceFlags flags = 0;
+
+	g_return_val_if_fail (SECRET_IS_ITEM (self), SECRET_ITEM_NONE);
+
+	g_mutex_lock (&self->pv->mutex);
+
+	if (self->pv->value)
+		flags |= SECRET_ITEM_LOAD_SECRET;
+
+	g_mutex_unlock (&self->pv->mutex);
+
+	return flags;
+
+}
+
 /**
  * secret_item_get_service:
  * @self: an item
@@ -948,35 +1059,63 @@ secret_item_delete_sync (SecretItem *self,
 SecretService *
 secret_item_get_service (SecretItem *self)
 {
-	g_return_val_if_fail (SECRET_IS_COLLECTION (self), NULL);
+	g_return_val_if_fail (SECRET_IS_ITEM (self), NULL);
 	return self->pv->service;
 }
 
 
+/**
+ * secret_item_get_secret:
+ * @self: an item
+ *
+ * Get the secret value of this item. If this item is locked or the secret
+ * has not yet been loaded then this will return %NULL.
+ *
+ * To load the secret call the secret_item_load_secret() method.
+ *
+ * Returns: (transfer full) (allow-none): the secret value which should be
+ *          released with secret_value_unref(), or %NULL
+ */
+SecretValue *
+secret_item_get_secret (SecretItem *self)
+{
+	SecretValue *value = NULL;
+
+	g_return_val_if_fail (SECRET_IS_ITEM (self), NULL);
+
+	g_mutex_lock (&self->pv->mutex);
+
+	if (self->pv->value)
+		value = secret_value_ref (self->pv->value);
+
+	g_mutex_unlock (&self->pv->mutex);
+
+	return value;
+}
+
+
 typedef struct {
 	GCancellable *cancellable;
-	SecretValue *value;
-} GetClosure;
+} LoadClosure;
 
 static void
-get_closure_free (gpointer data)
+load_closure_free (gpointer data)
 {
-	GetClosure *closure = data;
+	LoadClosure *closure = data;
 	g_clear_object (&closure->cancellable);
-	secret_value_unref (closure->value);
-	g_slice_free (GetClosure, closure);
+	g_slice_free (LoadClosure, closure);
 }
 
 static void
-on_item_get_secret (GObject *source,
-                    GAsyncResult *result,
-                    gpointer user_data)
+on_item_load_secret (GObject *source,
+                     GAsyncResult *result,
+                     gpointer user_data)
 {
 	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
 	SecretItem *self = SECRET_ITEM (g_async_result_get_source_object (user_data));
-	GetClosure *closure = g_simple_async_result_get_op_res_gpointer (res);
 	SecretSession *session;
 	GError *error = NULL;
+	SecretValue *value;
 	GVariant *retval;
 	GVariant *child;
 
@@ -986,43 +1125,50 @@ on_item_get_secret (GObject *source,
 		g_variant_unref (retval);
 
 		session = _secret_service_get_session (self->pv->service);
-		closure->value = _secret_session_decode_secret (session, child);
+		value = _secret_session_decode_secret (session, child);
 		g_variant_unref (child);
 
-		if (closure->value == NULL)
+		if (value == NULL) {
 			g_set_error (&error, SECRET_ERROR, SECRET_ERROR_PROTOCOL,
 			             _("Received invalid secret from the secret storage"));
+		} else {
+			_secret_item_set_cached_secret (self, value);
+			secret_value_unref (value);
+		}
 	}
 
-	if (error != NULL)
+	if (error != NULL) {
+		_secret_util_strip_remote_error (&error);
 		g_simple_async_result_take_error (res, error);
+	}
 
 	g_simple_async_result_complete (res);
 	g_object_unref (res);
 }
 
 static void
-on_get_ensure_session (GObject *source,
-                       GAsyncResult *result,
-                       gpointer user_data)
+on_load_ensure_session (GObject *source,
+                        GAsyncResult *result,
+                        gpointer user_data)
 {
 	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
 	SecretItem *self = SECRET_ITEM (g_async_result_get_source_object (user_data));
-	GetClosure *closure = g_simple_async_result_get_op_res_gpointer (res);
+	LoadClosure *load = g_simple_async_result_get_op_res_gpointer (res);
 	const gchar *session_path;
 	GError *error = NULL;
 
-	session_path = secret_service_ensure_session_finish (self->pv->service, result, &error);
+	secret_service_ensure_session_finish (self->pv->service, result, &error);
 	if (error != NULL) {
 		g_simple_async_result_take_error (res, error);
 		g_simple_async_result_complete (res);
 
 	} else {
+		session_path = secret_service_get_session_dbus_path (self->pv->service);
 		g_assert (session_path != NULL && session_path[0] != '\0');
 		g_dbus_proxy_call (G_DBUS_PROXY (self), "GetSecret",
 		                   g_variant_new ("(o)", session_path),
-		                   G_DBUS_CALL_FLAGS_NONE, -1, closure->cancellable,
-		                   on_item_get_secret, g_object_ref (res));
+		                   G_DBUS_CALL_FLAGS_NONE, -1, load->cancellable,
+		                   on_item_load_secret, g_object_ref (res));
 	}
 
 	g_object_unref (self);
@@ -1030,83 +1176,83 @@ on_get_ensure_session (GObject *source,
 }
 
 /**
- * secret_item_get_secret:
- * @self: an item
+ * secret_item_load_secret:
+ * @self: an item proxy
  * @cancellable: optional cancellation object
  * @callback: called when the operation completes
  * @user_data: data to pass to the callback
  *
- * Get the secret value of this item.
+ * Load the secret value of this item.
  *
  * Each item has a single secret which might be a password or some
  * other secret binary value.
  *
+ * This function will fail if the secret item is locked.
+ *
  * This function returns immediately and completes asynchronously.
  */
 void
-secret_item_get_secret (SecretItem *self,
-                        GCancellable *cancellable,
-                        GAsyncReadyCallback callback,
-                        gpointer user_data)
+secret_item_load_secret (SecretItem *self,
+                         GCancellable *cancellable,
+                         GAsyncReadyCallback callback,
+                         gpointer user_data)
 {
 	GSimpleAsyncResult *res;
-	GetClosure *closure;
+	LoadClosure *closure;
 
 	g_return_if_fail (SECRET_IS_ITEM (self));
 	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
 	res = g_simple_async_result_new (G_OBJECT (self), callback,
-	                                 user_data, secret_item_get_secret);
-	closure = g_slice_new0 (GetClosure);
+	                                 user_data, secret_item_load_secret);
+	closure = g_slice_new0 (LoadClosure);
 	closure->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
-	g_simple_async_result_set_op_res_gpointer (res, closure, get_closure_free);
+	g_simple_async_result_set_op_res_gpointer (res, closure, load_closure_free);
 
 	secret_service_ensure_session (self->pv->service, cancellable,
-	                               on_get_ensure_session,
+	                               on_load_ensure_session,
 	                               g_object_ref (res));
 
 	g_object_unref (res);
 }
 
 /**
- * secret_item_get_secret_finish:
- * @self: an item
+ * secret_item_load_secret_finish:
+ * @self: an item proxy
  * @result: asynchronous result passed to callback
  * @error: location to place error on failure
  *
- * Get the secret value of this item.
+ * Complete asynchronous operation to load the secret value of this item.
  *
- * Complete asynchronous operation to get the secret value of this item.
+ * The newly loaded secret value can be accessed by calling
+ * secret_item_get_secret().
  *
- * Returns: (transfer full): the newly allocated secret value in this
- *          item, which should be released with secret_value_unref()
+ * Returns: whether the secret item succesfully loaded or not
  */
-SecretValue *
-secret_item_get_secret_finish (SecretItem *self,
-                               GAsyncResult *result,
-                               GError **error)
+gboolean
+secret_item_load_secret_finish (SecretItem *self,
+                                GAsyncResult *result,
+                                GError **error)
 {
 	GSimpleAsyncResult *res;
-	GetClosure *closure;
 
 	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (self),
-	                      secret_item_get_secret), NULL);
+	                      secret_item_load_secret), FALSE);
 
 	res = G_SIMPLE_ASYNC_RESULT (result);
 	if (g_simple_async_result_propagate_error (res, error))
-		return NULL;
+		return FALSE;
 
-	closure = g_simple_async_result_get_op_res_gpointer (res);
-	return closure->value ? secret_value_ref (closure->value) : NULL;
+	return TRUE;
 }
 
 /**
- * secret_item_get_secret_sync:
+ * secret_item_load_secret_sync:
  * @self: an item
  * @cancellable: optional cancellation object
  * @error: location to place error on failure
  *
- * Get the secret value of this item.
+ * Load the secret value of this item.
  *
  * Each item has a single secret which might be a password or some
  * other secret binary value.
@@ -1114,16 +1260,15 @@ secret_item_get_secret_finish (SecretItem *self,
  * This function may block indefinetely. Use the asynchronous version
  * in user interface threads.
  *
- * Returns: (transfer full): the newly allocated secret value in this
- *          item, which should be released with secret_value_unref()
+ * Returns: whether the secret item succesfully loaded or not
  */
-SecretValue *
-secret_item_get_secret_sync (SecretItem *self,
-                             GCancellable *cancellable,
-                             GError **error)
+gboolean
+secret_item_load_secret_sync (SecretItem *self,
+                              GCancellable *cancellable,
+                              GError **error)
 {
 	SecretSync *sync;
-	SecretValue *value;
+	gboolean result;
 
 	g_return_val_if_fail (SECRET_IS_ITEM (self), FALSE);
 	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
@@ -1132,16 +1277,249 @@ secret_item_get_secret_sync (SecretItem *self,
 	sync = _secret_sync_new ();
 	g_main_context_push_thread_default (sync->context);
 
-	secret_item_get_secret (self, cancellable, _secret_sync_on_result, sync);
+	secret_item_load_secret (self, cancellable, _secret_sync_on_result, sync);
 
 	g_main_loop_run (sync->loop);
 
-	value = secret_item_get_secret_finish (self, sync->result, error);
+	result = secret_item_load_secret_finish (self, sync->result, error);
 
 	g_main_context_pop_thread_default (sync->context);
 	_secret_sync_free (sync);
 
-	return value;
+	return result;
+}
+
+typedef struct {
+	SecretService *service;
+	GCancellable *cancellable;
+	GVariant *in;
+	GHashTable *items;
+} LoadsClosure;
+
+static void
+loads_closure_free (gpointer data)
+{
+	LoadsClosure *loads = data;
+	if (loads->in)
+		g_variant_unref (loads->in);
+	if (loads->service)
+		g_object_unref (loads->service);
+	g_clear_object (&loads->cancellable);
+	g_slice_free (LoadsClosure, loads);
+}
+
+static void
+on_get_secrets_complete (GObject *source,
+                         GAsyncResult *result,
+                         gpointer user_data)
+{
+	GSimpleAsyncResult *async = G_SIMPLE_ASYNC_RESULT (user_data);
+	LoadsClosure *loads = g_simple_async_result_get_op_res_gpointer (async);
+	GHashTable *with_paths;
+	GError *error = NULL;
+	GHashTableIter iter;
+	const gchar *path;
+	SecretValue *value;
+	SecretItem *item;
+	GVariant *retval;
+
+	retval = g_dbus_proxy_call_finish (G_DBUS_PROXY (source), result, &error);
+	if (retval != NULL) {
+		with_paths = _secret_service_decode_get_secrets_all (loads->service, retval);
+		g_return_if_fail (with_paths != NULL);
+
+		g_hash_table_iter_init (&iter, with_paths);
+		while (g_hash_table_iter_next (&iter, (gpointer *)&path, (gpointer *)&value)) {
+			item = g_hash_table_lookup (loads->items, path);
+			if (item != NULL)
+				_secret_item_set_cached_secret (item, value);
+		}
+
+		g_hash_table_unref (with_paths);
+		g_variant_unref (retval);
+	}
+
+	if (error != NULL) {
+		_secret_util_strip_remote_error (&error);
+		g_simple_async_result_take_error (async, error);
+	}
+
+	g_simple_async_result_complete (async);
+	g_object_unref (async);
+}
+
+static void
+on_loads_secrets_session (GObject *source,
+                          GAsyncResult *result,
+                          gpointer user_data)
+{
+	GSimpleAsyncResult *async = G_SIMPLE_ASYNC_RESULT (user_data);
+	LoadsClosure *loads = g_simple_async_result_get_op_res_gpointer (async);
+	GError *error = NULL;
+	const gchar *session;
+
+	secret_service_ensure_session_finish (SECRET_SERVICE (source), result, &error);
+	if (error != NULL) {
+		g_simple_async_result_take_error (async, error);
+		g_simple_async_result_complete (async);
+
+	} else {
+		session = secret_service_get_session_dbus_path (SECRET_SERVICE (source));
+		g_dbus_proxy_call (G_DBUS_PROXY (source), "GetSecrets",
+		                   g_variant_new ("(@aoo)", loads->in, session),
+		                   G_DBUS_CALL_FLAGS_NO_AUTO_START, -1,
+		                   loads->cancellable, on_get_secrets_complete,
+		                   g_object_ref (async));
+	}
+
+	g_object_unref (async);
+}
+
+/**
+ * secret_item_load_secrets:
+ * @items: (element-type Secret.Item): the items to retrieve secrets for
+ * @cancellable: optional cancellation object
+ * @callback: called when the operation completes
+ * @user_data: data to pass to the callback
+ *
+ * Load the secret values for an secret items stored in the service.
+ *
+ * The @items must all have the same SecretItem::service property.
+ *
+ * This function returns immediately and completes asynchronously.
+ */
+void
+secret_item_load_secrets (GList *items,
+                          GCancellable *cancellable,
+                          GAsyncReadyCallback callback,
+                          gpointer user_data)
+{
+	GSimpleAsyncResult *async;
+	LoadsClosure *loads;
+	GPtrArray *paths;
+	const gchar *path;
+	GList *l;
+
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+
+	for (l = items; l != NULL; l = g_list_next (l))
+		g_return_if_fail (SECRET_IS_ITEM (l->data));
+
+	async = g_simple_async_result_new (NULL, callback, user_data,
+	                                   secret_item_load_secrets);
+	loads = g_slice_new0 (LoadsClosure);
+	loads->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
+	loads->items = g_hash_table_new_full (g_str_hash, g_str_equal,
+	                                      g_free, g_object_unref);
+
+	paths = g_ptr_array_new ();
+	for (l = items; l != NULL; l = g_list_next (l)) {
+		if (secret_item_get_locked (l->data))
+			continue;
+
+		if (loads->service == NULL) {
+			loads->service = secret_item_get_service (l->data);
+			if (loads->service)
+				g_object_ref (loads->service);
+		}
+
+		path = g_dbus_proxy_get_object_path (l->data);
+		g_hash_table_insert (loads->items, g_strdup (path), g_object_ref (l->data));
+		g_ptr_array_add (paths, (gpointer)path);
+	}
+
+	loads->in = g_variant_new_objv ((const gchar * const *)paths->pdata, paths->len);
+	g_variant_ref_sink (loads->in);
+
+	g_ptr_array_free (paths, TRUE);
+	g_simple_async_result_set_op_res_gpointer (async, loads, loads_closure_free);
+
+	if (loads->service) {
+		secret_service_ensure_session (loads->service, cancellable,
+		                               on_loads_secrets_session,
+		                               g_object_ref (async));
+	} else {
+		g_simple_async_result_complete_in_idle (async);
+	}
+
+	g_object_unref (async);
+}
+
+/**
+ * secret_item_load_secrets_finish:
+ * @result: asynchronous result passed to callback
+ * @error: location to place an error on failure
+ *
+ * Complete asynchronous operation to load the secret values for
+ * secret items stored in the service.
+ *
+ * Items that are locked will not have their secrets loaded.
+ *
+ * Returns: whether the operation succeded or not
+ */
+gboolean
+secret_item_load_secrets_finish (GAsyncResult *result,
+                                 GError **error)
+{
+	GSimpleAsyncResult *async;
+
+	g_return_val_if_fail (g_simple_async_result_is_valid (result, NULL,
+	                      secret_item_load_secrets), FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	async = G_SIMPLE_ASYNC_RESULT (result);
+	if (g_simple_async_result_propagate_error (async, error))
+		return FALSE;
+
+	return TRUE;
+}
+
+/**
+ * secret_item_load_secrets_sync:
+ * @items: (element-type Secret.Item): the items to retrieve secrets for
+ * @cancellable: optional cancellation object
+ * @error: location to place an error on failure
+ *
+ * Load the secret values for an secret items stored in the service.
+ *
+ * The @items must all have the same SecretItem::service property.
+ *
+ * This method may block indefinitely and should not be used in user interface
+ * threads.
+ *
+ * Items that are locked will not have their secrets loaded.
+ *
+ * Returns: whether the operation succeded or not
+ */
+gboolean
+secret_item_load_secrets_sync (GList *items,
+                               GCancellable *cancellable,
+                               GError **error)
+{
+	SecretSync *sync;
+	gboolean ret;
+	GList *l;
+
+	for (l = items; l != NULL; l = g_list_next (l))
+		g_return_val_if_fail (SECRET_IS_ITEM (l->data), FALSE);
+
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	sync = _secret_sync_new ();
+	g_main_context_push_thread_default (sync->context);
+
+	secret_item_load_secrets (items, cancellable,
+	                          _secret_sync_on_result, sync);
+
+	g_main_loop_run (sync->loop);
+
+	ret = secret_item_load_secrets_finish (sync->result, error);
+
+	g_main_context_pop_thread_default (sync->context);
+	_secret_sync_free (sync);
+
+	return ret;
 }
 
 typedef struct {
@@ -1152,10 +1530,10 @@ typedef struct {
 static void
 set_closure_free (gpointer data)
 {
-	GetClosure *closure = data;
-	g_clear_object (&closure->cancellable);
-	secret_value_unref (closure->value);
-	g_slice_free (GetClosure, closure);
+	SetClosure *set = data;
+	g_clear_object (&set->cancellable);
+	secret_value_unref (set->value);
+	g_slice_free (SetClosure, set);
 }
 
 static void
@@ -1164,17 +1542,24 @@ on_item_set_secret (GObject *source,
                     gpointer user_data)
 {
 	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
+	SecretItem *self = SECRET_ITEM (g_async_result_get_source_object (user_data));
+	SetClosure *set = g_simple_async_result_get_op_res_gpointer (res);
 	GError *error = NULL;
 	GVariant *retval;
 
 	retval = g_dbus_proxy_call_finish (G_DBUS_PROXY (source), result, &error);
 
-	if (error != NULL)
+	if (error == NULL) {
+		_secret_item_set_cached_secret (self, set->value);
+	} else {
+		_secret_util_strip_remote_error (&error);
 		g_simple_async_result_take_error (res, error);
+	}
 	if (retval != NULL)
 		g_variant_unref (retval);
 
 	g_simple_async_result_complete (res);
+	g_object_unref (self);
 	g_object_unref (res);
 }
 
@@ -1350,7 +1735,7 @@ secret_item_get_attributes (SecretItem *self)
 	variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (self), "Attributes");
 	g_return_val_if_fail (variant != NULL, NULL);
 
-	attributes = _secret_util_attributes_for_variant (variant);
+	attributes = _secret_attributes_for_variant (variant);
 	g_variant_unref (variant);
 
 	return attributes;
@@ -1383,7 +1768,7 @@ secret_item_set_attributes (SecretItem *self,
 	g_return_if_fail (attributes != NULL);
 
 	_secret_util_set_property (G_DBUS_PROXY (self), "Attributes",
-	                           _secret_util_variant_for_attributes (attributes, NULL),
+	                           _secret_attributes_to_variant (attributes, NULL),
 	                           secret_item_set_attributes, cancellable,
 	                           callback, user_data);
 }
@@ -1438,7 +1823,7 @@ secret_item_set_attributes_sync (SecretItem *self,
 	g_return_val_if_fail (attributes != NULL, FALSE);
 
 	return _secret_util_set_property_sync (G_DBUS_PROXY (self), "Attributes",
-	                                       _secret_util_variant_for_attributes (attributes, NULL),
+	                                       _secret_attributes_to_variant (attributes, NULL),
 	                                       cancellable, error);
 }
 
