@@ -14,6 +14,7 @@
 
 #include "config.h"
 
+#include "secret-attributes.h"
 #include "secret-password.h"
 #include "secret-private.h"
 #include "secret-value.h"
@@ -41,80 +42,10 @@
  * Stability: Stable
  */
 
-typedef struct {
-	const SecretSchema *schema;
-	GHashTable *attributes;
-	gchar *collection_path;
-	gchar *label;
-	SecretValue *value;
-	GCancellable *cancellable;
-	gboolean created;
-} StoreClosure;
-
-static void
-store_closure_free (gpointer data)
-{
-	StoreClosure *closure = data;
-	_secret_schema_unref_if_nonstatic (closure->schema);
-	g_hash_table_unref (closure->attributes);
-	g_free (closure->collection_path);
-	g_free (closure->label);
-	secret_value_unref (closure->value);
-	g_clear_object (&closure->cancellable);
-	g_slice_free (StoreClosure, closure);
-}
-
-static void
-on_store_complete (GObject *source,
-                   GAsyncResult *result,
-                   gpointer user_data)
-{
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	StoreClosure *closure = g_simple_async_result_get_op_res_gpointer (res);
-	GError *error = NULL;
-
-	closure->created = secret_service_store_finish (SECRET_SERVICE (source),
-	                                                result, &error);
-	if (error != NULL)
-		g_simple_async_result_take_error (res, error);
-
-	g_simple_async_result_complete (res);
-	g_object_unref (res);
-}
-
-static void
-on_store_connected (GObject *source,
-                    GAsyncResult *result,
-                    gpointer user_data)
-{
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	StoreClosure *closure = g_simple_async_result_get_op_res_gpointer (res);
-	SecretService *service;
-	GError *error = NULL;
-
-	service = secret_service_get_finish (result, &error);
-	if (error == NULL) {
-		secret_service_storev (service, closure->schema,
-		                       closure->attributes,
-		                       closure->collection_path,
-		                       closure->label, closure->value,
-		                       closure->cancellable,
-		                       on_store_complete,
-		                       g_object_ref (res));
-		g_object_unref (service);
-
-	} else {
-		g_simple_async_result_take_error (res, error);
-		g_simple_async_result_complete (res);
-	}
-
-	g_object_unref (res);
-}
-
 /**
- * secret_password_store:
+ * secret_password_store: (skip)
  * @schema: the schema for attributes
- * @collection_path: (allow-none): the D-Bus object path of the collection where to store the secret
+ * @collection: (allow-none): a collection alias, or D-Bus object path of the collection where to store the secret
  * @label: label for the secret
  * @password: the null-terminated password to store
  * @cancellable: optional cancellation object
@@ -132,7 +63,7 @@ on_store_connected (GObject *source,
  * If the attributes match a secret item already stored in the collection, then
  * the item will be updated with these new values.
  *
- * If @collection_path is %NULL, then the default collection will be
+ * If @collection is %NULL, then the default collection will be
  * used. Use #SECRET_COLLECTION_SESSION to store the password in the session
  * collection, which doesn't get stored across login sessions.
  *
@@ -140,7 +71,7 @@ on_store_connected (GObject *source,
  */
 void
 secret_password_store (const SecretSchema *schema,
-                       const gchar *collection_path,
+                       const gchar *collection,
                        const gchar *label,
                        const gchar *password,
                        GCancellable *cancellable,
@@ -157,10 +88,10 @@ secret_password_store (const SecretSchema *schema,
 	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
 	va_start (va, user_data);
-	attributes = _secret_util_attributes_for_varargs (schema, va);
+	attributes = secret_attributes_buildv (schema, va);
 	va_end (va);
 
-	secret_password_storev (schema, attributes, collection_path, label, password,
+	secret_password_storev (schema, attributes, collection, label, password,
 	                        cancellable, callback, user_data);
 
 	g_hash_table_unref (attributes);
@@ -170,7 +101,7 @@ secret_password_store (const SecretSchema *schema,
  * secret_password_storev:
  * @schema: the schema for attributes
  * @attributes: (element-type utf8 utf8): the attribute keys and values
- * @collection_path: (allow-none): the D-Bus object path of the collection where to store the secret
+ * @collection: (allow-none): a collection alias, or D-Bus object path of the collection where to store the secret
  * @label: label for the secret
  * @password: the null-terminated password to store
  * @cancellable: optional cancellation object
@@ -184,7 +115,7 @@ secret_password_store (const SecretSchema *schema,
  * If the attributes match a secret item already stored in the collection, then
  * the item will be updated with these new values.
  *
- * If @collection_path is %NULL, then the default collection will be
+ * If @collection is %NULL, then the default collection will be
  * used. Use #SECRET_COLLECTION_SESSION to store the password in the session
  * collection, which doesn't get stored across login sessions.
  *
@@ -195,15 +126,14 @@ secret_password_store (const SecretSchema *schema,
 void
 secret_password_storev (const SecretSchema *schema,
                         GHashTable *attributes,
-                        const gchar *collection_path,
+                        const gchar *collection,
                         const gchar *label,
                         const gchar *password,
                         GCancellable *cancellable,
                         GAsyncReadyCallback callback,
                         gpointer user_data)
 {
-	GSimpleAsyncResult *res;
-	StoreClosure *closure;
+	SecretValue *value;
 
 	g_return_if_fail (schema != NULL);
 	g_return_if_fail (label != NULL);
@@ -212,24 +142,15 @@ secret_password_storev (const SecretSchema *schema,
 	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
 	/* Warnings raised already */
-	if (!_secret_util_attributes_validate (schema, attributes))
+	if (!_secret_attributes_validate (schema, attributes))
 		return;
 
-	res = g_simple_async_result_new (NULL, callback, user_data,
-	                                 secret_password_storev);
-	closure = g_slice_new0 (StoreClosure);
-	closure->schema = _secret_schema_ref_if_nonstatic (schema);
-	closure->collection_path = g_strdup (collection_path);
-	closure->label = g_strdup (label);
-	closure->value = secret_value_new (password, -1, "text/plain");
-	closure->attributes = _secret_util_attributes_copy (attributes);
-	closure->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
-	g_simple_async_result_set_op_res_gpointer (res, closure, store_closure_free);
+	value = secret_value_new (password, -1, "text/plain");
 
-	secret_service_get (SECRET_SERVICE_OPEN_SESSION, cancellable,
-	                    on_store_connected, g_object_ref (res));
+	secret_service_store (NULL, schema, attributes, collection,
+	                      label, value, cancellable, callback, user_data);
 
-	g_object_unref (res);
+	secret_value_unref (value);
 }
 
 /**
@@ -245,25 +166,14 @@ gboolean
 secret_password_store_finish (GAsyncResult *result,
                               GError **error)
 {
-	GSimpleAsyncResult *res;
-	StoreClosure *closure;
-
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-	g_return_val_if_fail (g_simple_async_result_is_valid (result, NULL,
-	                      secret_password_storev), FALSE);
-
-	res = G_SIMPLE_ASYNC_RESULT (result);
-	if (g_simple_async_result_propagate_error (res, error))
-		return FALSE;
-
-	closure = g_simple_async_result_get_op_res_gpointer (res);
-	return closure->created;
+	return secret_service_store_finish (NULL, result, error);
 }
 
 /**
  * secret_password_store_sync:
  * @schema: the schema for attributes
- * @collection_path: (allow-none): the D-Bus object path of the collection where to store the secret
+ * @collection: (allow-none): a collection alias, or D-Bus object path of the collection where to store the secret
  * @label: label for the secret
  * @password: the null-terminated password to store
  * @cancellable: optional cancellation object
@@ -280,7 +190,7 @@ secret_password_store_finish (GAsyncResult *result,
  * If the attributes match a secret item already stored in the collection, then
  * the item will be updated with these new values.
  *
- * If @collection_path is %NULL, then the default collection will be
+ * If @collection is %NULL, then the default collection will be
  * used. Use #SECRET_COLLECTION_SESSION to store the password in the session
  * collection, which doesn't get stored across login sessions.
  *
@@ -291,7 +201,7 @@ secret_password_store_finish (GAsyncResult *result,
  */
 gboolean
 secret_password_store_sync (const SecretSchema *schema,
-                            const gchar *collection_path,
+                            const gchar *collection,
                             const gchar *label,
                             const gchar *password,
                             GCancellable *cancellable,
@@ -309,10 +219,10 @@ secret_password_store_sync (const SecretSchema *schema,
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
 	va_start (va, error);
-	attributes = _secret_util_attributes_for_varargs (schema, va);
+	attributes = secret_attributes_buildv (schema, va);
 	va_end (va);
 
-	ret = secret_password_storev_sync (schema, attributes, collection_path,
+	ret = secret_password_storev_sync (schema, attributes, collection,
 	                                   label, password, cancellable, error);
 
 	g_hash_table_unref (attributes);
@@ -323,7 +233,7 @@ secret_password_store_sync (const SecretSchema *schema,
  * secret_password_storev_sync:
  * @schema: the schema for attributes
  * @attributes: (element-type utf8 utf8): the attribute keys and values
- * @collection_path: (allow-none): the D-Bus object path of the collection where to store the secret
+ * @collection: (allow-none): a collection alias, or D-Bus object path of the collection where to store the secret
  * @label: label for the secret
  * @password: the null-terminated password to store
  * @cancellable: optional cancellation object
@@ -336,7 +246,7 @@ secret_password_store_sync (const SecretSchema *schema,
  * If the attributes match a secret item already stored in the collection, then
  * the item will be updated with these new values.
  *
- * If @collection_path is %NULL, then the default collection will be
+ * If @collection is %NULL, then the default collection will be
  * used. Use #SECRET_COLLECTION_SESSION to store the password in the session
  * collection, which doesn't get stored across login sessions.
  *
@@ -350,7 +260,7 @@ secret_password_store_sync (const SecretSchema *schema,
 gboolean
 secret_password_storev_sync (const SecretSchema *schema,
                              GHashTable *attributes,
-                             const gchar *collection_path,
+                             const gchar *collection,
                              const gchar *label,
                              const gchar *password,
                              GCancellable *cancellable,
@@ -367,13 +277,13 @@ secret_password_storev_sync (const SecretSchema *schema,
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
 	/* Warnings raised already */
-	if (!_secret_util_attributes_validate (schema, attributes))
+	if (!_secret_attributes_validate (schema, attributes))
 		return FALSE;
 
 	sync = _secret_sync_new ();
 	g_main_context_push_thread_default (sync->context);
 
-	secret_password_storev (schema, attributes, collection_path, label, password,
+	secret_password_storev (schema, attributes, collection, label, password,
 	                        cancellable, _secret_sync_on_result, sync);
 
 	g_main_loop_run (sync->loop);
@@ -386,25 +296,6 @@ secret_password_storev_sync (const SecretSchema *schema,
 	return ret;
 }
 
-typedef struct {
-	GCancellable *cancellable;
-	GHashTable *attributes;
-	SecretValue *value;
-	const SecretSchema *schema;
-} LookupClosure;
-
-static void
-lookup_closure_free (gpointer data)
-{
-	LookupClosure *closure = data;
-	_secret_schema_unref_if_nonstatic (closure->schema);
-	g_clear_object (&closure->cancellable);
-	g_hash_table_unref (closure->attributes);
-	if (closure->value)
-		secret_value_unref (closure->value);
-	g_slice_free (LookupClosure, closure);
-}
-
 /**
  * secret_password_lookup: (skip)
  * @schema: the schema for the attributes
@@ -438,7 +329,7 @@ secret_password_lookup (const SecretSchema *schema,
 	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
 	va_start (va, user_data);
-	attributes = _secret_util_attributes_for_varargs (schema, va);
+	attributes = secret_attributes_buildv (schema, va);
 	va_end (va);
 
 	secret_password_lookupv (schema, attributes, cancellable,
@@ -447,50 +338,6 @@ secret_password_lookup (const SecretSchema *schema,
 	g_hash_table_unref (attributes);
 }
 
-static void
-on_lookup_complete (GObject *source,
-                    GAsyncResult *result,
-                    gpointer user_data)
-{
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	LookupClosure *closure = g_simple_async_result_get_op_res_gpointer (res);
-	GError *error = NULL;
-
-	closure->value = secret_service_lookup_finish (SECRET_SERVICE (source),
-	                                               result, &error);
-
-	if (error != NULL)
-		g_simple_async_result_take_error (res, error);
-
-	g_simple_async_result_complete (res);
-	g_object_unref (res);
-}
-
-static void
-on_lookup_connected (GObject *source,
-                     GAsyncResult *result,
-                     gpointer user_data)
-{
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	LookupClosure *closure = g_simple_async_result_get_op_res_gpointer (res);
-	SecretService *service;
-	GError *error = NULL;
-
-	service = secret_service_get_finish (result, &error);
-	if (error != NULL) {
-		g_simple_async_result_take_error (res, error);
-		g_simple_async_result_complete (res);
-
-	} else {
-		secret_service_lookupv (service, closure->schema, closure->attributes,
-		                        closure->cancellable, on_lookup_complete,
-		                        g_object_ref (res));
-		g_object_unref (service);
-	}
-
-	g_object_unref (res);
-}
-
 /**
  * secret_password_lookupv:
  * @schema: the schema for attributes
@@ -516,29 +363,16 @@ secret_password_lookupv (const SecretSchema *schema,
                          GAsyncReadyCallback callback,
                          gpointer user_data)
 {
-	GSimpleAsyncResult *res;
-	LookupClosure *closure;
-
 	g_return_if_fail (schema != NULL);
 	g_return_if_fail (attributes != NULL);
 	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
 	/* Warnings raised already */
-	if (!_secret_util_attributes_validate (schema, attributes))
+	if (!_secret_attributes_validate (schema, attributes))
 		return;
 
-	res = g_simple_async_result_new (NULL, callback, user_data,
-	                                 secret_password_lookupv);
-	closure = g_slice_new0 (LookupClosure);
-	closure->schema = _secret_schema_ref_if_nonstatic (schema);
-	closure->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
-	closure->attributes = _secret_util_attributes_copy (attributes);
-	g_simple_async_result_set_op_res_gpointer (res, closure, lookup_closure_free);
-
-	secret_service_get (SECRET_SERVICE_OPEN_SESSION, cancellable,
-	                    on_lookup_connected, g_object_ref (res));
-
-	g_object_unref (res);
+	secret_service_lookup (NULL, schema, attributes,
+	                       cancellable, callback, user_data);
 }
 
 /**
@@ -555,25 +389,15 @@ gchar *
 secret_password_lookup_nonpageable_finish (GAsyncResult *result,
                                            GError **error)
 {
-	GSimpleAsyncResult *res;
-	LookupClosure *closure;
-	gchar *password = NULL;
+	SecretValue *value;
 
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
-	g_return_val_if_fail (g_simple_async_result_is_valid (result, NULL,
-	                      secret_password_lookupv), NULL);
 
-	res = G_SIMPLE_ASYNC_RESULT (result);
-	if (g_simple_async_result_propagate_error (res, error))
+	value = secret_service_lookup_finish (NULL, result, error);
+	if (value == NULL)
 		return NULL;
 
-	closure = g_simple_async_result_get_op_res_gpointer (res);
-	if (closure->value == NULL)
-		return NULL;
-
-	password = _secret_value_unref_to_password (closure->value);
-	closure->value = NULL;
-	return password;
+	return _secret_value_unref_to_password (value);
 }
 
 /**
@@ -590,25 +414,15 @@ gchar *
 secret_password_lookup_finish (GAsyncResult *result,
                                GError **error)
 {
-	GSimpleAsyncResult *res;
-	LookupClosure *closure;
-	gchar *string = NULL;
+	SecretValue *value;
 
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
-	g_return_val_if_fail (g_simple_async_result_is_valid (result, NULL,
-	                      secret_password_lookupv), NULL);
 
-	res = G_SIMPLE_ASYNC_RESULT (result);
-	if (g_simple_async_result_propagate_error (res, error))
+	value = secret_service_lookup_finish (NULL, result, error);
+	if (value == NULL)
 		return NULL;
 
-	closure = g_simple_async_result_get_op_res_gpointer (res);
-	if (closure->value == NULL)
-		return NULL;
-
-	string = _secret_value_unref_to_string (closure->value);
-	closure->value = NULL;
-	return string;
+	return _secret_value_unref_to_string (value);
 }
 
 /**
@@ -648,7 +462,7 @@ secret_password_lookup_sync (const SecretSchema *schema,
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	va_start (va, error);
-	attributes = _secret_util_attributes_for_varargs (schema, va);
+	attributes = secret_attributes_buildv (schema, va);
 	va_end (va);
 
 	password = secret_password_lookupv_sync (schema, attributes,
@@ -696,7 +510,7 @@ secret_password_lookup_nonpageable_sync (const SecretSchema *schema,
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	va_start (va, error);
-	attributes = _secret_util_attributes_for_varargs (schema, va);
+	attributes = secret_attributes_buildv (schema, va);
 	va_end (va);
 
 	password = secret_password_lookupv_nonpageable_sync (schema, attributes,
@@ -741,7 +555,7 @@ secret_password_lookupv_nonpageable_sync (const SecretSchema *schema,
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* Warnings raised already */
-	if (!_secret_util_attributes_validate (schema, attributes))
+	if (!_secret_attributes_validate (schema, attributes))
 		return FALSE;
 
 	sync = _secret_sync_new ();
@@ -796,7 +610,7 @@ secret_password_lookupv_sync (const SecretSchema *schema,
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* Warnings raised already */
-	if (!_secret_util_attributes_validate (schema, attributes))
+	if (!_secret_attributes_validate (schema, attributes))
 		return FALSE;
 
 	sync = _secret_sync_new ();
@@ -815,23 +629,6 @@ secret_password_lookupv_sync (const SecretSchema *schema,
 	return string;
 }
 
-typedef struct {
-	GCancellable *cancellable;
-	GHashTable *attributes;
-	gboolean deleted;
-	const SecretSchema *schema;
-} DeleteClosure;
-
-static void
-delete_closure_free (gpointer data)
-{
-	DeleteClosure *closure = data;
-	g_clear_object (&closure->cancellable);
-	g_hash_table_unref (closure->attributes);
-	_secret_schema_unref_if_nonstatic (closure->schema);
-	g_slice_free (DeleteClosure, closure);
-}
-
 /**
  * secret_password_remove:
  * @schema: the schema for the attributes
@@ -865,7 +662,7 @@ secret_password_remove (const SecretSchema *schema,
 	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
 	va_start (va, user_data);
-	attributes = _secret_util_attributes_for_varargs (schema, va);
+	attributes = secret_attributes_buildv (schema, va);
 	va_end (va);
 
 	secret_password_removev (schema, attributes, cancellable,
@@ -874,48 +671,6 @@ secret_password_remove (const SecretSchema *schema,
 	g_hash_table_unref (attributes);
 }
 
-static void
-on_delete_complete (GObject *source,
-                    GAsyncResult *result,
-                    gpointer user_data)
-{
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	DeleteClosure *closure = g_simple_async_result_get_op_res_gpointer (res);
-	GError *error = NULL;
-
-	closure->deleted = secret_service_remove_finish (SECRET_SERVICE (source),
-	                                                 result, &error);
-	if (error != NULL)
-		g_simple_async_result_take_error (res, error);
-	g_simple_async_result_complete (res);
-
-	g_object_unref (res);
-}
-
-static void
-on_delete_connect (GObject *source,
-                   GAsyncResult *result,
-                   gpointer user_data)
-{
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	DeleteClosure *closure = g_simple_async_result_get_op_res_gpointer (res);
-	SecretService *service;
-	GError *error = NULL;
-
-	service = secret_service_get_finish (result, &error);
-	if (error == NULL) {
-		secret_service_removev (service, closure->schema, closure->attributes,
-		                        closure->cancellable, on_delete_complete,
-		                        g_object_ref (res));
-		g_object_unref (service);
-
-	} else {
-		g_simple_async_result_take_error (res, error);
-		g_simple_async_result_complete (res);
-	}
-
-	g_object_unref (res);
-}
 
 /**
  * secret_password_removev:
@@ -942,29 +697,16 @@ secret_password_removev (const SecretSchema *schema,
                          GAsyncReadyCallback callback,
                          gpointer user_data)
 {
-	GSimpleAsyncResult *res;
-	DeleteClosure *closure;
-
 	g_return_if_fail (schema != NULL);
 	g_return_if_fail (attributes != NULL);
 	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
 	/* Warnings raised already */
-	if (!_secret_util_attributes_validate (schema, attributes))
+	if (!_secret_attributes_validate (schema, attributes))
 		return;
 
-	res = g_simple_async_result_new (NULL, callback, user_data,
-	                                 secret_password_removev);
-	closure = g_slice_new0 (DeleteClosure);
-	closure->schema = _secret_schema_ref_if_nonstatic (schema);
-	closure->attributes = _secret_util_attributes_copy (attributes);
-	closure->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
-	g_simple_async_result_set_op_res_gpointer (res, closure, delete_closure_free);
-
-	secret_service_get (SECRET_SERVICE_NONE, cancellable,
-	                    on_delete_connect, g_object_ref (res));
-
-	g_object_unref (res);
+	secret_service_remove (NULL, schema, attributes,
+	                       cancellable, callback, user_data);
 }
 
 /**
@@ -981,19 +723,8 @@ gboolean
 secret_password_remove_finish (GAsyncResult *result,
                                GError **error)
 {
-	DeleteClosure *closure;
-	GSimpleAsyncResult *res;
-
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-	g_return_val_if_fail (g_simple_async_result_is_valid (result, NULL,
-	                      secret_password_removev), FALSE);
-
-	res = G_SIMPLE_ASYNC_RESULT (result);
-	if (g_simple_async_result_propagate_error (res, error))
-		return FALSE;
-
-	closure = g_simple_async_result_get_op_res_gpointer (res);
-	return closure->deleted;
+	return secret_service_remove_finish (NULL, result, error);
 }
 
 /**
@@ -1032,7 +763,7 @@ secret_password_remove_sync (const SecretSchema* schema,
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
 	va_start (va, error);
-	attributes = _secret_util_attributes_for_varargs (schema, va);
+	attributes = secret_attributes_buildv (schema, va);
 	va_end (va);
 
 	result = secret_password_removev_sync (schema, attributes,
@@ -1078,7 +809,7 @@ secret_password_removev_sync (const SecretSchema *schema,
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
 	/* Warnings raised already */
-	if (!_secret_util_attributes_validate (schema, attributes))
+	if (!_secret_attributes_validate (schema, attributes))
 		return FALSE;
 
 	sync = _secret_sync_new ();
