@@ -19,6 +19,7 @@
 #include "secret-dbus-generated.h"
 #include "secret-enum-types.h"
 #include "secret-item.h"
+#include "secret-paths.h"
 #include "secret-private.h"
 #include "secret-service.h"
 #include "secret-types.h"
@@ -53,7 +54,7 @@
  * represent those collections while initializing a #SecretService then pass
  * the %SECRET_SERVICE_LOAD_COLLECTIONS flag to the secret_service_get() or
  * secret_service_new() functions. In order to establish a session on an already
- * existing #SecretService, use the secret_service_ensure_collections() function.
+ * existing #SecretService, use the secret_service_load_collections() function.
  * To access the list of collections use secret_service_get_collections().
  *
  * Certain actions on the Secret Service require user prompting to complete,
@@ -102,6 +103,7 @@
 
 EGG_SECURE_GLIB_DEFINITIONS ();
 
+GQuark _secret_error_quark = 0;
 static const gchar *default_bus_name = SECRET_SERVICE_BUS_NAME;
 
 enum {
@@ -123,6 +125,7 @@ struct _SecretServicePrivate {
 
 G_LOCK_DEFINE (service_instance);
 static gpointer service_instance = NULL;
+static guint service_watch = 0;
 
 static GInitableIface *secret_service_initable_parent_iface = NULL;
 
@@ -137,6 +140,84 @@ G_DEFINE_TYPE_WITH_CODE (SecretService, secret_service, G_TYPE_DBUS_PROXY,
                          G_IMPLEMENT_INTERFACE (G_TYPE_ASYNC_INITABLE, secret_service_async_initable_iface);
 );
 
+static SecretService *
+service_get_instance (void)
+{
+	SecretService *instance = NULL;
+
+	G_LOCK (service_instance);
+	if (service_instance != NULL)
+		instance = g_object_ref (service_instance);
+	G_UNLOCK (service_instance);
+
+	return instance;
+}
+
+static gboolean
+service_uncache_instance (SecretService *which)
+{
+	SecretService *instance = NULL;
+	guint watch = 0;
+	gboolean matched = FALSE;
+
+	G_LOCK (service_instance);
+	if (which == NULL || service_instance == which) {
+		instance = service_instance;
+		service_instance = NULL;
+		watch = service_watch;
+		service_watch = 0;
+		matched = TRUE;
+	}
+	G_UNLOCK (service_instance);
+
+	if (instance != NULL)
+		g_object_unref (instance);
+	if (watch != 0)
+		g_bus_unwatch_name (watch);
+
+	return matched;
+}
+
+static void
+on_service_instance_vanished (GDBusConnection *connection,
+                              const gchar *name,
+                              gpointer user_data)
+{
+	if (!service_uncache_instance (user_data)) {
+		g_warning ("Global default SecretService instance out of sync "
+		           "with the watch for its DBus name");
+	}
+}
+
+static void
+service_cache_instance (SecretService *instance)
+{
+	GDBusProxy *proxy;
+	guint watch;
+
+	g_object_ref (instance);
+	proxy = G_DBUS_PROXY (instance);
+	watch = g_bus_watch_name_on_connection (g_dbus_proxy_get_connection (proxy),
+	                                        g_dbus_proxy_get_name (proxy),
+	                                        G_BUS_NAME_WATCHER_FLAGS_AUTO_START,
+	                                        NULL, on_service_instance_vanished,
+	                                        instance, NULL);
+
+	G_LOCK (service_instance);
+	if (service_instance == NULL) {
+		service_instance = instance;
+		instance = NULL;
+		service_watch = watch;
+		watch = 0;
+	}
+	G_UNLOCK (service_instance);
+
+	if (instance != NULL)
+		g_object_unref (instance);
+	if (watch != 0)
+		g_bus_unwatch_name (watch);
+}
+
 static void
 secret_service_init (SecretService *self)
 {
@@ -205,6 +286,7 @@ secret_service_finalize (GObject *obj)
 	if (self->pv->collections)
 		g_hash_table_destroy (self->pv->collections);
 	g_clear_object (&self->pv->cancellable);
+	g_mutex_clear (&self->pv->mutex);
 
 	G_OBJECT_CLASS (secret_service_parent_class)->finalize (obj);
 }
@@ -304,7 +386,7 @@ handle_property_changed (SecretService *self,
 		g_mutex_unlock (&self->pv->mutex);
 
 		if (perform)
-			secret_service_ensure_collections (self, self->pv->cancellable, NULL, NULL);
+			secret_service_load_collections (self, self->pv->cancellable, NULL, NULL);
 	}
 
 	g_variant_unref (value);
@@ -354,7 +436,7 @@ secret_service_signal (GDBusProxy *proxy,
 
 	/* A new collection was added, add it to the Collections property */
 	if (g_str_equal (signal_name, SECRET_SIGNAL_COLLECTION_CREATED)) {
-		g_variant_get (parameters, "@o", &value);
+		g_variant_get (parameters, "(@o)", &value);
 		g_variant_builder_init (&builder, G_VARIANT_TYPE ("ao"));
 		g_variant_iter_init (&iter, paths);
 		while ((path = g_variant_iter_next_value (&iter)) != NULL) {
@@ -374,7 +456,7 @@ secret_service_signal (GDBusProxy *proxy,
 
 	/* A collection was deleted, remove it from the Collections property */
 	} else if (g_str_equal (signal_name, SECRET_SIGNAL_COLLECTION_DELETED)) {
-		g_variant_get (parameters, "@o", &value);
+		g_variant_get (parameters, "(@o)", &value);
 		g_variant_builder_init (&builder, G_VARIANT_TYPE ("ao"));
 		g_variant_iter_init (&iter, paths);
 		while ((path = g_variant_iter_next_value (&iter)) != NULL) {
@@ -390,7 +472,7 @@ secret_service_signal (GDBusProxy *proxy,
 
 	/* The collection changed, update it */
 	} else if (g_str_equal (signal_name, SECRET_SIGNAL_COLLECTION_CHANGED)) {
-		g_variant_get (parameters, "&o", &collection_path);
+		g_variant_get (parameters, "(&o)", &collection_path);
 
 		g_mutex_lock (&self->pv->mutex);
 
@@ -403,8 +485,10 @@ secret_service_signal (GDBusProxy *proxy,
 
 		g_mutex_unlock (&self->pv->mutex);
 
-		secret_collection_refresh (collection);
-		g_object_unref (collection);
+		if (collection) {
+			secret_collection_refresh (collection);
+			g_object_unref (collection);
+		}
 	}
 
 	g_variant_unref (paths);
@@ -451,7 +535,7 @@ secret_service_class_init (SecretServiceClass *klass)
 	 *
 	 * To load the collections, specify the %SECRET_SERVICE_LOAD_COLLECTIONS
 	 * initialization flag when calling the secret_service_get() or
-	 * secret_service_new() functions. Or call the secret_service_ensure_collections()
+	 * secret_service_new() functions. Or call the secret_service_load_collections()
 	 * method.
 	 */
 	g_object_class_install_property (object_class, PROP_COLLECTIONS,
@@ -459,6 +543,9 @@ secret_service_class_init (SecretServiceClass *klass)
 	                                 _secret_list_get_type (), G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	g_type_class_add_private (klass, sizeof (SecretServicePrivate));
+
+	/* Initialize this error domain, registers dbus errors */
+	_secret_error_quark = secret_error_get_quark ();
 }
 
 typedef struct {
@@ -485,22 +572,22 @@ service_ensure_for_flags_sync (SecretService *self,
 			return FALSE;
 
 	if (flags & SECRET_SERVICE_LOAD_COLLECTIONS)
-		if (!secret_service_ensure_collections_sync (self, cancellable, error))
+		if (!secret_service_load_collections_sync (self, cancellable, error))
 			return FALSE;
 
 	return TRUE;
 }
 
 static void
-on_ensure_collections (GObject *source,
-                       GAsyncResult *result,
-                       gpointer user_data)
+on_load_collections (GObject *source,
+                     GAsyncResult *result,
+                     gpointer user_data)
 {
 	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
 	SecretService *self = SECRET_SERVICE (source);
 	GError *error = NULL;
 
-	if (!secret_service_ensure_collections_finish (self, result, &error))
+	if (!secret_service_load_collections_finish (self, result, &error))
 		g_simple_async_result_take_error (res, error);
 
 	g_simple_async_result_complete (res);
@@ -522,8 +609,8 @@ on_ensure_session (GObject *source,
 		g_simple_async_result_complete (res);
 
 	} else if (closure->flags & SECRET_SERVICE_LOAD_COLLECTIONS) {
-		secret_service_ensure_collections (self, closure->cancellable,
-		                                   on_ensure_collections, g_object_ref (res));
+		secret_service_load_collections (self, closure->cancellable,
+		                                 on_load_collections, g_object_ref (res));
 
 	} else {
 		g_simple_async_result_complete_in_idle (res);
@@ -546,8 +633,8 @@ service_ensure_for_flags_async (SecretService *self,
 		                               on_ensure_session, g_object_ref (res));
 
 	else if (closure->flags & SECRET_SERVICE_LOAD_COLLECTIONS)
-		secret_service_ensure_collections (self, closure->cancellable,
-		                                   on_ensure_collections, g_object_ref (res));
+		secret_service_load_collections (self, closure->cancellable,
+		                                 on_load_collections, g_object_ref (res));
 
 	else
 		g_simple_async_result_complete_in_idle (res);
@@ -589,10 +676,9 @@ on_init_base (GObject *source,
 		g_simple_async_result_take_error (res, error);
 		g_simple_async_result_complete (res);
 	} else {
-
+		service_ensure_for_flags_async (self, self->pv->init_flags, res);
 	}
 
-	service_ensure_for_flags_async (self, self->pv->init_flags, res);
 	g_object_unref (res);
 }
 
@@ -650,18 +736,6 @@ _secret_service_set_default_bus_name (const gchar *bus_name)
 	default_bus_name = bus_name;
 }
 
-static void
-on_service_instance_gone (gpointer user_data,
-                          GObject *where_the_object_was)
-{
-	G_LOCK (service_instance);
-
-		g_assert (service_instance == where_the_object_was);
-		service_instance = NULL;
-
-	G_UNLOCK (service_instance);
-}
-
 /**
  * secret_service_get:
  * @flags: flags for which service functionality to ensure is initialized
@@ -687,10 +761,7 @@ secret_service_get (SecretServiceFlags flags,
 	GSimpleAsyncResult *res;
 	InitClosure *closure;
 
-	G_LOCK (service_instance);
-	if (service_instance != NULL)
-		service = g_object_ref (service_instance);
-	G_UNLOCK (service_instance);
+	service = service_get_instance ();
 
 	/* Create a whole new service */
 	if (service == NULL) {
@@ -752,15 +823,8 @@ secret_service_get_finish (GAsyncResult *result,
 	/* Creating a whole new service */
 	} else {
 		service = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), result, error);
-
-		if (service) {
-			G_LOCK (service_instance);
-			if (service_instance == NULL) {
-				service_instance = service;
-				g_object_weak_ref (G_OBJECT (service), on_service_instance_gone, NULL);
-			}
-			G_UNLOCK (service_instance);
-		}
+		if (service)
+			service_cache_instance (SECRET_SERVICE (service));
 	}
 
 	if (source_object)
@@ -797,10 +861,7 @@ secret_service_get_sync (SecretServiceFlags flags,
 {
 	SecretService *service = NULL;
 
-	G_LOCK (service_instance);
-	if (service_instance != NULL)
-		service = g_object_ref (service_instance);
-	G_UNLOCK (service_instance);
+	service = service_get_instance ();
 
 	if (service == NULL) {
 		service = g_initable_new (SECRET_TYPE_SERVICE, cancellable, error,
@@ -813,14 +874,8 @@ secret_service_get_sync (SecretServiceFlags flags,
 		                          "flags", flags,
 		                          NULL);
 
-		if (service != NULL) {
-			G_LOCK (service_instance);
-			if (service_instance == NULL) {
-				service_instance = service;
-				g_object_weak_ref (G_OBJECT (service), on_service_instance_gone, NULL);
-			}
-			G_UNLOCK (service_instance);
-		}
+		if (service != NULL)
+			service_cache_instance (service);
 
 	} else {
 		if (!service_ensure_for_flags_sync (service, flags, cancellable, error)) {
@@ -832,8 +887,28 @@ secret_service_get_sync (SecretServiceFlags flags,
 	return service;
 }
 
+/**
+ * secret_service_disconnect:
+ *
+ * Disconnect the default #SecretService proxy returned by secret_service_get()
+ * and secret_service_get_sync().
+ *
+ * It is not necessary to call this function, but you may choose to do so at
+ * program exit. It is useful for testing that memory is not leaked.
+ *
+ * This function is safe to call at any time. But if other objects in this
+ * library are still referenced, then this will not result in all memory
+ * being freed.
+ */
+void
+secret_service_disconnect (void)
+{
+	service_uncache_instance (NULL);
+}
+
 /**
  * secret_service_new:
+ * @service_gtype: the GType of the new secret service
  * @service_bus_name: (allow-none): the D-Bus service name of the secret service
  * @flags: flags for which service functionality to ensure is initialized
  * @cancellable: optional cancellation object
@@ -844,6 +919,9 @@ secret_service_get_sync (SecretServiceFlags flags,
  *
  * This function is rarely used, see secret_service_get() instead.
  *
+ * The @service_gtype argument should be set to %SECRET_TYPE_SERVICE or a the type
+ * of a derived class.
+ *
  * If @flags contains any flags of which parts of the secret service to
  * ensure are initialized, then those will be initialized before returning.
  *
@@ -852,18 +930,20 @@ secret_service_get_sync (SecretServiceFlags flags,
  * This method will return immediately and complete asynchronously.
  */
 void
-secret_service_new (const gchar *service_bus_name,
+secret_service_new (GType service_gtype,
+                    const gchar *service_bus_name,
                     SecretServiceFlags flags,
                     GCancellable *cancellable,
                     GAsyncReadyCallback callback,
                     gpointer user_data)
 {
 	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+	g_return_if_fail (g_type_is_a (service_gtype, SECRET_TYPE_SERVICE));
 
 	if (service_bus_name == NULL)
 		service_bus_name = default_bus_name;
 
-	g_async_initable_new_async (SECRET_TYPE_SERVICE, G_PRIORITY_DEFAULT,
+	g_async_initable_new_async (service_gtype, G_PRIORITY_DEFAULT,
 	                            cancellable, callback, user_data,
 	                            "g-flags", G_DBUS_PROXY_FLAGS_NONE,
 	                            "g-interface-info", _secret_gen_service_interface_info (),
@@ -909,6 +989,7 @@ secret_service_new_finish (GAsyncResult *result,
 
 /**
  * secret_service_new_sync:
+ * @service_gtype: the GType of the new secret service
  * @service_bus_name: (allow-none): the D-Bus service name of the secret service
  * @flags: flags for which service functionality to ensure is initialized
  * @cancellable: optional cancellation object
@@ -918,6 +999,9 @@ secret_service_new_finish (GAsyncResult *result,
  *
  * This function is rarely used, see secret_service_get_sync() instead.
  *
+ * The @service_gtype argument should be set to %SECRET_TYPE_SERVICE or a the
+ * type of a derived class.
+ *
  * If @flags contains any flags of which parts of the secret service to
  * ensure are initialized, then those will be initialized before returning.
  *
@@ -930,17 +1014,19 @@ secret_service_new_finish (GAsyncResult *result,
  *          should be released with g_object_unref().
  */
 SecretService *
-secret_service_new_sync (const gchar *service_bus_name,
+secret_service_new_sync (GType service_gtype,
+                         const gchar *service_bus_name,
                          SecretServiceFlags flags,
                          GCancellable *cancellable,
                          GError **error)
 {
 	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (g_type_is_a (service_gtype, SECRET_TYPE_SERVICE), NULL);
 
 	if (service_bus_name == NULL)
 		service_bus_name = default_bus_name;
 
-	return g_initable_new (SECRET_TYPE_SERVICE, cancellable, error,
+	return g_initable_new (service_gtype, cancellable, error,
 	                       "g-flags", G_DBUS_PROXY_FLAGS_NONE,
 	                       "g-interface-info", _secret_gen_service_interface_info (),
 	                       "g-name", service_bus_name,
@@ -958,7 +1044,7 @@ secret_service_new_sync (const gchar *service_bus_name,
  * Get the flags representing what features of the #SecretService proxy
  * have been initialized.
  *
- * Use secret_service_ensure_session() or secret_service_ensure_collections()
+ * Use secret_service_ensure_session() or secret_service_load_collections()
  * to initialize further features and change the flags.
  *
  * Returns: the flags for features initialized
@@ -991,7 +1077,7 @@ secret_service_get_flags (SecretService *self)
  *
  * If the %SECRET_SERVICE_LOAD_COLLECTIONS flag was not specified when
  * initializing #SecretService proxy object, then this method will return
- * %NULL. Use secret_service_ensure_collections() to load the collections.
+ * %NULL. Use secret_service_load_collections() to load the collections.
  *
  * Returns: (transfer full) (element-type Secret.Collection) (allow-none): a
  *          list of the collections in the secret service
@@ -1119,7 +1205,7 @@ secret_service_get_session_algorithms (SecretService *self)
 }
 
 /**
- * secret_service_get_session_path:
+ * secret_service_get_session_dbus_path:
  * @self: the secret service proxy
  *
  * Get the D-Bus object path of the session object being used to transfer
@@ -1132,7 +1218,7 @@ secret_service_get_session_algorithms (SecretService *self)
  *          session
  */
 const gchar *
-secret_service_get_session_path (SecretService *self)
+secret_service_get_session_dbus_path (SecretService *self)
 {
 	SecretSession *session;
 	const gchar *path;
@@ -1201,24 +1287,24 @@ secret_service_ensure_session (SecretService *self,
  * Finish an asynchronous operation to ensure that the #SecretService proxy
  * has established a session with the Secret Service.
  *
- * Returns: the path of the established session
+ * Returns: whether a session is established or not
  */
-const gchar *
+gboolean
 secret_service_ensure_session_finish (SecretService *self,
                                       GAsyncResult *result,
                                       GError **error)
 {
-	g_return_val_if_fail (SECRET_IS_SERVICE (self), NULL);
-	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+	g_return_val_if_fail (SECRET_IS_SERVICE (self), FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
 	if (!g_simple_async_result_is_valid (result, G_OBJECT (self),
 	                                     secret_service_ensure_session)) {
 		if (!_secret_session_open_finish (result, error))
-			return NULL;
+			return FALSE;
 	}
 
-	g_return_val_if_fail (self->pv->session != NULL, NULL);
-	return secret_service_get_session_path (self);
+	g_return_val_if_fail (self->pv->session != NULL, FALSE);
+	return TRUE;
 }
 
 /**
@@ -1238,19 +1324,19 @@ secret_service_ensure_session_finish (SecretService *self,
  * This method may block indefinitely and should not be used in user interface
  * threads.
  *
- * Returns: the path of the established session
+ * Returns: whether a session is established or not
  */
-const gchar *
+gboolean
 secret_service_ensure_session_sync (SecretService *self,
                                     GCancellable *cancellable,
                                     GError **error)
 {
 	SecretSync *sync;
-	const gchar *path;
+	gboolean ret;
 
-	g_return_val_if_fail (SECRET_IS_SERVICE (self), NULL);
-	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
-	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+	g_return_val_if_fail (SECRET_IS_SERVICE (self), FALSE);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
 	sync = _secret_sync_new ();
 	g_main_context_push_thread_default (sync->context);
@@ -1260,12 +1346,12 @@ secret_service_ensure_session_sync (SecretService *self,
 
 	g_main_loop_run (sync->loop);
 
-	path = secret_service_ensure_session_finish (self, sync->result, error);
+	ret = secret_service_ensure_session_finish (self, sync->result, error);
 
 	g_main_context_pop_thread_default (sync->context);
 	_secret_sync_free (sync);
 
-	return path;
+	return ret;
 }
 
 static SecretCollection *
@@ -1304,6 +1390,8 @@ service_update_collections (SecretService *self,
 
 	if (previous != NULL)
 		g_hash_table_unref (previous);
+
+	g_object_notify (G_OBJECT (self), "collections");
 }
 
 typedef struct {
@@ -1342,13 +1430,13 @@ on_ensure_collection (GObject *source,
 
 	closure->collections_loading--;
 
-	collection = secret_collection_new_finish (result, &error);
+	collection = secret_collection_new_for_dbus_path_finish (result, &error);
 
 	if (error != NULL)
 		g_simple_async_result_take_error (res, error);
 
 	if (collection != NULL) {
-		path = g_dbus_proxy_get_object_path (G_DBUS_PROXY (self));
+		path = g_dbus_proxy_get_object_path (G_DBUS_PROXY (collection));
 		g_hash_table_insert (closure->collections, g_strdup (path), collection);
 	}
 
@@ -1362,7 +1450,7 @@ on_ensure_collection (GObject *source,
 }
 
 /**
- * secret_service_ensure_collections:
+ * secret_service_load_collections:
  * @self: the secret service
  * @cancellable: optional cancellation object
  * @callback: called when the operation completes
@@ -1379,10 +1467,10 @@ on_ensure_collection (GObject *source,
  * This method will return immediately and complete asynchronously.
  */
 void
-secret_service_ensure_collections (SecretService *self,
-                                   GCancellable *cancellable,
-                                   GAsyncReadyCallback callback,
-                                   gpointer user_data)
+secret_service_load_collections (SecretService *self,
+                                 GCancellable *cancellable,
+                                 GAsyncReadyCallback callback,
+                                 gpointer user_data)
 {
 	EnsureClosure *closure;
 	SecretCollection *collection;
@@ -1398,7 +1486,7 @@ secret_service_ensure_collections (SecretService *self,
 	g_return_if_fail (paths != NULL);
 
 	res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
-	                                 secret_service_ensure_collections);
+	                                 secret_service_load_collections);
 	closure = g_slice_new0 (EnsureClosure);
 	closure->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
 	closure->collections = collections_table_new ();
@@ -1410,8 +1498,8 @@ secret_service_ensure_collections (SecretService *self,
 
 		/* No such collection yet create a new one */
 		if (collection == NULL) {
-			secret_collection_new (self, path, cancellable,
-			                       on_ensure_collection, g_object_ref (res));
+			secret_collection_new_for_dbus_path (self, path, SECRET_COLLECTION_LOAD_ITEMS,
+			                                     cancellable, on_ensure_collection, g_object_ref (res));
 			closure->collections_loading++;
 		} else {
 			g_hash_table_insert (closure->collections, g_strdup (path), collection);
@@ -1428,7 +1516,7 @@ secret_service_ensure_collections (SecretService *self,
 }
 
 /**
- * secret_service_ensure_collections_finish:
+ * secret_service_load_collections_finish:
  * @self: the secret service
  * @result: the asynchronous result passed to the callback
  * @error: location to place an error on failure
@@ -1439,14 +1527,14 @@ secret_service_ensure_collections (SecretService *self,
  * Returns: whether the load was successful or not
  */
 gboolean
-secret_service_ensure_collections_finish (SecretService *self,
-                                          GAsyncResult *result,
-                                          GError **error)
+secret_service_load_collections_finish (SecretService *self,
+                                        GAsyncResult *result,
+                                        GError **error)
 {
 	g_return_val_if_fail (SECRET_IS_SERVICE (self), FALSE);
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (self),
-	                      secret_service_ensure_collections), FALSE);
+	                      secret_service_load_collections), FALSE);
 
 	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
 		return FALSE;
@@ -1455,7 +1543,7 @@ secret_service_ensure_collections_finish (SecretService *self,
 }
 
 /**
- * secret_service_ensure_collections_sync:
+ * secret_service_load_collections_sync:
  * @self: the secret service
  * @cancellable: optional cancellation object
  * @error: location to place an error on failure
@@ -1474,9 +1562,9 @@ secret_service_ensure_collections_finish (SecretService *self,
  * Returns: whether the load was successful or not
  */
 gboolean
-secret_service_ensure_collections_sync (SecretService *self,
-                                        GCancellable *cancellable,
-                                        GError **error)
+secret_service_load_collections_sync (SecretService *self,
+                                      GCancellable *cancellable,
+                                      GError **error)
 {
 	SecretCollection *collection;
 	GHashTable *collections;
@@ -1500,7 +1588,9 @@ secret_service_ensure_collections_sync (SecretService *self,
 
 		/* No such collection yet create a new one */
 		if (collection == NULL) {
-			collection = secret_collection_new_sync (self, path, cancellable, error);
+			collection = secret_collection_new_for_dbus_path_sync (self, path,
+			                                                       SECRET_COLLECTION_LOAD_ITEMS,
+			                                                       cancellable, error);
 			if (collection == NULL) {
 				ret = FALSE;
 				break;
@@ -1632,89 +1722,3 @@ secret_service_prompt_finish (SecretService *self,
 
 	return (klass->prompt_finish) (self, result, return_type, error);
 }
-
-GVariant *
-secret_service_prompt_path_sync (SecretService *self,
-                                 const gchar *prompt_path,
-                                 GCancellable *cancellable,
-                                 const GVariantType *return_type,
-                                 GError **error)
-{
-	SecretPrompt *prompt;
-	GVariant *retval;
-
-	g_return_val_if_fail (SECRET_IS_SERVICE (self), NULL);
-	g_return_val_if_fail (prompt_path != NULL, NULL);
-	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
-	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
-
-	prompt = _secret_prompt_instance (self, prompt_path);
-	retval = secret_service_prompt_sync (self, prompt, cancellable, return_type, error);
-	g_object_unref (prompt);
-
-	return retval;
-}
-
-/**
- * secret_service_prompt_path:
- * @self: the secret service
- * @prompt_path: the D-Bus object path of the prompt
- * @cancellable: optional cancellation object
- * @callback: called when the operation completes
- * @user_data: data to be passed to the callback
- *
- * Perform prompting for a #SecretPrompt.
- *
- * This function is called by other parts of this library to handle prompts
- * for the various actions that can require prompting.
- *
- * Override the #SecretServiceClass <literal>prompt_async</literal> virtual method
- * to change the behavior of the propmting. The default behavior is to simply
- * run secret_prompt_perform() on the prompt.
- */
-void
-secret_service_prompt_path (SecretService *self,
-                            const gchar *prompt_path,
-                            GCancellable *cancellable,
-                            GAsyncReadyCallback callback,
-                            gpointer user_data)
-{
-	SecretPrompt *prompt;
-
-	g_return_if_fail (SECRET_IS_SERVICE (self));
-	g_return_if_fail (prompt_path != NULL);
-	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
-
-	prompt = _secret_prompt_instance (self, prompt_path);
-	secret_service_prompt (self, prompt, cancellable, callback, user_data);
-	g_object_unref (prompt);
-}
-
-/**
- * secret_service_prompt_path_finish:
- * @self: the secret service
- * @result: the asynchronous result passed to the callback
- * @return_type: the variant type of the prompt result
- * @error: location to place an error on failure
- *
- * Complete asynchronous operation to perform prompting for a #SecretPrompt.
- *
- * Returns a variant result if the prompt was completed and not dismissed. The
- * type of result depends on the action the prompt is completing, and is defined
- * in the Secret Service DBus API specification.
- *
- * Returns: (transfer full): %NULL if the prompt was dismissed or an error occurred,
- *          a variant result if the prompt was successful
- */
-GVariant *
-secret_service_prompt_path_finish (SecretService *self,
-                                   GAsyncResult *result,
-                                   const GVariantType *return_type,
-                                   GError **error)
-{
-	g_return_val_if_fail (SECRET_IS_SERVICE (self), NULL);
-	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), NULL);
-	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
-
-	return secret_service_prompt_finish (self, result, return_type, error);
-}
