@@ -29,6 +29,9 @@ GQuark          secret_error_get_quark      (void) G_GNUC_CONST;
 
 typedef enum {
 	SECRET_ERROR_PROTOCOL = 1,
+	SECRET_ERROR_IS_LOCKED = 2,
+	SECRET_ERROR_NO_SUCH_OBJECT = 3,
+	SECRET_ERROR_ALREADY_EXISTS = 4,
 } SecretError;
 
 typedef struct _SecretCollection  SecretCollection;
@@ -37,9 +40,16 @@ typedef struct _SecretPrompt      SecretPrompt;
 typedef struct _SecretService     SecretService;
 typedef struct _SecretValue       SecretValue;
 
-#define SECRET_COLLECTION_DEFAULT "/org/freedesktop/secrets/aliases/default"
+#define SECRET_COLLECTION_DEFAULT "default"
 
-#define SECRET_COLLECTION_SESSION "/org/freedesktop/secrets/aliases/session"
+#define SECRET_COLLECTION_SESSION "session"
+
+typedef enum {
+	SECRET_SEARCH_NONE = 0,
+	SECRET_SEARCH_ALL = 1 << 1,
+	SECRET_SEARCH_UNLOCK = 1 << 2,
+	SECRET_SEARCH_LOAD_SECRETS = 1 << 3,
+} SecretSearchFlags;
 
 G_END_DECLS
 
