@@ -25,9 +25,11 @@
 
 #define __SECRET_INSIDE_HEADER__
 
+#include <secret/secret-attributes.h>
 #include <secret/secret-collection.h>
 #include <secret/secret-enum-types.h>
 #include <secret/secret-item.h>
+#include <secret/secret-paths.h>
 #include <secret/secret-prompt.h>
 #include <secret/secret-service.h>
 #include <secret/secret-value.h>
