@@ -4,10 +4,12 @@
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published
- * by the Free Software Foundation; either version 2 of the licence or (at
+ * by the Free Software Foundation; either version 2.1 of the licence or (at
  * your option) any later version.
  *
  * See the included COPYING file for more information.
+ *
+ * Author: Stef Walter <stefw@gnome.org>
  */
 
 #ifndef __SECRET_H__
@@ -19,6 +21,7 @@
 
 #include <secret/secret-password.h>
 #include <secret/secret-schema.h>
+#include <secret/secret-schemas.h>
 #include <secret/secret-types.h>
 
 #undef __SECRET_INSIDE_HEADER__
