@@ -18,6 +18,7 @@
 #include "secret-collection.h"
 #include "secret-item.h"
 #include "secret-service.h"
+#include "secret-paths.h"
 #include "secret-private.h"
 
 #include "mock-service.h"
@@ -52,6 +53,7 @@ teardown (Test *test,
           gconstpointer unused)
 {
 	g_object_unref (test->service);
+	secret_service_disconnect ();
 	egg_assert_not_object (test->service);
 
 	mock_service_stop ();
@@ -89,7 +91,7 @@ test_new_sync (Test *test,
 	GError *error = NULL;
 	SecretItem *item;
 
-	item = secret_item_new_sync (test->service, item_path, NULL, &error);
+	item = secret_item_new_for_dbus_path_sync (test->service, item_path, SECRET_ITEM_NONE, NULL, &error);
 	g_assert_no_error (error);
 
 	g_assert_cmpstr (g_dbus_proxy_get_object_path (G_DBUS_PROXY (item)), ==, item_path);
@@ -105,7 +107,7 @@ test_new_sync_noexist (Test *test,
 	GError *error = NULL;
 	SecretItem *item;
 
-	item = secret_item_new_sync (test->service, item_path, NULL, &error);
+	item = secret_item_new_for_dbus_path_sync (test->service, item_path, SECRET_ITEM_NONE, NULL, &error);
 	g_assert_error (error, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD);
 	g_assert (item == NULL);
 }
@@ -119,12 +121,13 @@ test_new_async (Test *test,
 	GError *error = NULL;
 	SecretItem *item;
 
-	secret_item_new (test->service, item_path, NULL, on_async_result, &result);
+	secret_item_new_for_dbus_path (test->service, item_path, SECRET_ITEM_NONE,
+	                               NULL, on_async_result, &result);
 	g_assert (result == NULL);
 
 	egg_test_wait ();
 
-	item = secret_item_new_finish (result, &error);
+	item = secret_item_new_for_dbus_path_finish (result, &error);
 	g_assert_no_error (error);
 	g_object_unref (result);
 
@@ -142,12 +145,13 @@ test_new_async_noexist (Test *test,
 	GError *error = NULL;
 	SecretItem *item;
 
-	secret_item_new (test->service, item_path, NULL, on_async_result, &result);
+	secret_item_new_for_dbus_path (test->service, item_path, SECRET_ITEM_NONE,
+	                               NULL, on_async_result, &result);
 	g_assert (result == NULL);
 
 	egg_test_wait ();
 
-	item = secret_item_new_finish (result, &error);
+	item = secret_item_new_for_dbus_path_finish (result, &error);
 	g_assert_error (error, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD);
 	g_assert (item == NULL);
 	g_object_unref (result);
@@ -164,7 +168,8 @@ test_create_sync (Test *test,
 	GHashTable *attributes;
 	SecretValue *value;
 
-	collection = secret_collection_new_sync (test->service, collection_path, NULL, &error);
+	collection = secret_collection_new_for_dbus_path_sync (test->service, collection_path,
+	                                                       SECRET_COLLECTION_NONE, NULL, &error);
 	g_assert_no_error (error);
 
 	attributes = g_hash_table_new (g_str_hash, g_str_equal);
@@ -202,7 +207,8 @@ test_create_async (Test *test,
 	GHashTable *attributes;
 	SecretValue *value;
 
-	collection = secret_collection_new_sync (test->service, collection_path, NULL, &error);
+	collection = secret_collection_new_for_dbus_path_sync (test->service, collection_path,
+	                                                       SECRET_COLLECTION_NONE, NULL, &error);
 	g_assert_no_error (error);
 
 	attributes = g_hash_table_new (g_str_hash, g_str_equal);
@@ -248,7 +254,7 @@ test_properties (Test *test,
 	gboolean locked;
 	gchar *label;
 
-	item = secret_item_new_sync (test->service, item_path, NULL, &error);
+	item = secret_item_new_for_dbus_path_sync (test->service, item_path, SECRET_ITEM_NONE, NULL, &error);
 	g_assert_no_error (error);
 
 	g_assert (secret_item_get_locked (item) == FALSE);
@@ -304,7 +310,7 @@ test_set_label_sync (Test *test,
 	gboolean ret;
 	gchar *label;
 
-	item = secret_item_new_sync (test->service, item_path, NULL, &error);
+	item = secret_item_new_for_dbus_path_sync (test->service, item_path, SECRET_ITEM_NONE, NULL, &error);
 	g_assert_no_error (error);
 
 	label = secret_item_get_label (item);
@@ -333,7 +339,7 @@ test_set_label_async (Test *test,
 	gboolean ret;
 	gchar *label;
 
-	item = secret_item_new_sync (test->service, item_path, NULL, &error);
+	item = secret_item_new_for_dbus_path_sync (test->service, item_path, SECRET_ITEM_NONE, NULL, &error);
 	g_assert_no_error (error);
 
 	label = secret_item_get_label (item);
@@ -362,13 +368,18 @@ test_set_label_prop (Test *test,
                      gconstpointer unused)
 {
 	const gchar *item_path = "/org/freedesktop/secrets/collection/english/1";
+	GAsyncResult *result = NULL;
 	GError *error = NULL;
 	SecretItem *item;
 	guint sigs = 2;
 	gchar *label;
 
-	item = secret_item_new_sync (test->service, item_path, NULL, &error);
+	secret_item_new_for_dbus_path (test->service, item_path, SECRET_ITEM_NONE, NULL, on_async_result, &result);
+	g_assert (result == NULL);
+	egg_test_wait ();
+	item = secret_item_new_for_dbus_path_finish (result, &error);
 	g_assert_no_error (error);
+	g_object_unref (result);
 
 	label = secret_item_get_label (item);
 	g_assert_cmpstr (label, ==, "Item One");
@@ -397,7 +408,7 @@ test_set_attributes_sync (Test *test,
 	gboolean ret;
 	GHashTable *attributes;
 
-	item = secret_item_new_sync (test->service, item_path, NULL, &error);
+	item = secret_item_new_for_dbus_path_sync (test->service, item_path, SECRET_ITEM_NONE, NULL, &error);
 	g_assert_no_error (error);
 
 	attributes = secret_item_get_attributes (item);
@@ -435,7 +446,7 @@ test_set_attributes_async (Test *test,
 	SecretItem *item;
 	gboolean ret;
 
-	item = secret_item_new_sync (test->service, item_path, NULL, &error);
+	item = secret_item_new_for_dbus_path_sync (test->service, item_path, SECRET_ITEM_NONE, NULL, &error);
 	g_assert_no_error (error);
 
 	attributes = secret_item_get_attributes (item);
@@ -472,13 +483,18 @@ test_set_attributes_prop (Test *test,
                           gconstpointer unused)
 {
 	const gchar *item_path = "/org/freedesktop/secrets/collection/english/1";
+	GAsyncResult *result = NULL;
 	GError *error = NULL;
 	SecretItem *item;
 	GHashTable *attributes;
 	guint sigs = 2;
 
-	item = secret_item_new_sync (test->service, item_path, NULL, &error);
+	secret_item_new_for_dbus_path (test->service, item_path, SECRET_ITEM_NONE, NULL, on_async_result, &result);
+	g_assert (result == NULL);
+	egg_test_wait ();
+	item = secret_item_new_for_dbus_path_finish (result, &error);
 	g_assert_no_error (error);
+	g_object_unref (result);
 
 	attributes = secret_item_get_attributes (item);
 	g_assert_cmpstr (g_hash_table_lookup (attributes, "string"), ==, "one");
@@ -508,21 +524,28 @@ test_set_attributes_prop (Test *test,
 }
 
 static void
-test_get_secret_sync (Test *test,
-                      gconstpointer unused)
+test_load_secret_sync (Test *test,
+                       gconstpointer unused)
 {
 	const gchar *item_path = "/org/freedesktop/secrets/collection/english/1";
 	GError *error = NULL;
 	SecretItem *item;
 	SecretValue *value;
 	gconstpointer data;
+	gboolean ret;
 	gsize length;
 
-	item = secret_item_new_sync (test->service, item_path, NULL, &error);
+	item = secret_item_new_for_dbus_path_sync (test->service, item_path, SECRET_ITEM_NONE, NULL, &error);
 	g_assert_no_error (error);
 
-	value = secret_item_get_secret_sync (item, NULL, &error);
+	value = secret_item_get_secret (item);
+	g_assert (value == NULL);
+
+	ret = secret_item_load_secret_sync (item, NULL, &error);
 	g_assert_no_error (error);
+	g_assert (ret == TRUE);
+
+	value = secret_item_get_secret (item);
 	g_assert (value != NULL);
 
 	data = secret_value_get (value, &length);
@@ -534,8 +557,8 @@ test_get_secret_sync (Test *test,
 }
 
 static void
-test_get_secret_async (Test *test,
-                       gconstpointer unused)
+test_load_secret_async (Test *test,
+                        gconstpointer unused)
 {
 	const gchar *item_path = "/org/freedesktop/secrets/collection/english/1";
 	GAsyncResult *result = NULL;
@@ -543,21 +566,28 @@ test_get_secret_async (Test *test,
 	SecretItem *item;
 	SecretValue *value;
 	gconstpointer data;
+	gboolean ret;
 	gsize length;
 
-	item = secret_item_new_sync (test->service, item_path, NULL, &error);
+	item = secret_item_new_for_dbus_path_sync (test->service, item_path, SECRET_ITEM_NONE, NULL, &error);
 	g_assert_no_error (error);
 
-	secret_item_get_secret (item, NULL, on_async_result, &result);
+	value = secret_item_get_secret (item);
+	g_assert (value == NULL);
+
+	secret_item_load_secret (item, NULL, on_async_result, &result);
 	g_assert (result == NULL);
 
 	egg_test_wait ();
 
-	value = secret_item_get_secret_finish (item, result, &error);
+	ret = secret_item_load_secret_finish (item, result, &error);
 	g_assert_no_error (error);
-	g_assert (value != NULL);
+	g_assert (ret == TRUE);
 	g_object_unref (result);
 
+	value = secret_item_get_secret (item);
+	g_assert (value != NULL);
+
 	data = secret_value_get (value, &length);
 	egg_assert_cmpmem (data, length, ==, "111", 3);
 
@@ -575,22 +605,29 @@ test_set_secret_sync (Test *test,
 	SecretItem *item;
 	gconstpointer data;
 	SecretValue *value;
-	gsize length;
+	SecretValue *check;
 	gboolean ret;
+	gsize length;
 
 	value = secret_value_new ("Sinking", -1, "strange/content-type");
 
-	item = secret_item_new_sync (test->service, item_path, NULL, &error);
+	item = secret_item_new_for_dbus_path_sync (test->service, item_path, SECRET_ITEM_NONE, NULL, &error);
 	g_assert_no_error (error);
 
 	ret = secret_item_set_secret_sync (item, value, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (ret == TRUE);
 
+	check = secret_item_get_secret (item);
+	g_assert (check == value);
+	secret_value_unref (check);
 	secret_value_unref (value);
 
-	value = secret_item_get_secret_sync (item, NULL, &error);
+	ret = secret_item_load_secret_sync (item, NULL, &error);
 	g_assert_no_error (error);
+	g_assert (ret == TRUE);
+
+	value = secret_item_get_secret (item);
 	g_assert (value != NULL);
 
 	data = secret_value_get (value, &length);
@@ -601,6 +638,119 @@ test_set_secret_sync (Test *test,
 	g_object_unref (item);
 }
 
+static void
+test_secrets_sync (Test *test,
+                   gconstpointer used)
+{
+	const gchar *path_item_one = "/org/freedesktop/secrets/collection/english/1";
+	const gchar *path_item_two = "/org/freedesktop/secrets/collection/english/2";
+	const gchar *path_item_three = "/org/freedesktop/secrets/collection/spanish/10";
+
+	SecretValue *value;
+	GError *error = NULL;
+	const gchar *password;
+	SecretItem *item_one, *item_two, *item_three;
+	GList *items = NULL;
+	gboolean ret;
+	gsize length;
+
+	item_one = secret_item_new_for_dbus_path_sync (test->service, path_item_one, SECRET_ITEM_NONE, NULL, &error);
+	item_two = secret_item_new_for_dbus_path_sync (test->service, path_item_two, SECRET_ITEM_NONE, NULL, &error);
+	item_three = secret_item_new_for_dbus_path_sync (test->service, path_item_three, SECRET_ITEM_NONE, NULL, &error);
+
+	items = g_list_append (items, item_one);
+	items = g_list_append (items, item_two);
+	items = g_list_append (items, item_three);
+
+	ret = secret_item_load_secrets_sync (items, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (ret == TRUE);
+
+	value = secret_item_get_secret (item_one);
+	g_assert (value != NULL);
+	password = secret_value_get (value, &length);
+	g_assert_cmpuint (length, ==, 3);
+	g_assert_cmpstr (password, ==, "111");
+	secret_value_unref (value);
+
+	value = secret_item_get_secret (item_two);
+	g_assert (value != NULL);
+	password = secret_value_get (value, &length);
+	g_assert_cmpuint (length, ==, 3);
+	g_assert_cmpstr (password, ==, "222");
+	secret_value_unref (value);
+
+	value = secret_item_get_secret (item_three);
+	g_assert (value == NULL);
+
+	g_list_free_full (items, g_object_unref);
+}
+
+static void
+test_secrets_async (Test *test,
+                              gconstpointer used)
+{
+	const gchar *path_item_one = "/org/freedesktop/secrets/collection/english/1";
+	const gchar *path_item_two = "/org/freedesktop/secrets/collection/english/2";
+	const gchar *path_item_three = "/org/freedesktop/secrets/collection/spanish/10";
+
+	SecretValue *value;
+	GError *error = NULL;
+	const gchar *password;
+	GAsyncResult *result = NULL;
+	SecretItem *item_one, *item_two, *item_three;
+	GList *items = NULL;
+	gsize length;
+	gboolean ret;
+
+	item_one = secret_item_new_for_dbus_path_sync (test->service, path_item_one, SECRET_ITEM_NONE, NULL, &error);
+	g_assert_no_error (error);
+
+	item_two = secret_item_new_for_dbus_path_sync (test->service, path_item_two, SECRET_ITEM_NONE, NULL, &error);
+	g_assert_no_error (error);
+
+	item_three = secret_item_new_for_dbus_path_sync (test->service, path_item_three, SECRET_ITEM_NONE, NULL, &error);
+	g_assert_no_error (error);
+
+
+	items = g_list_append (items, item_one);
+	items = g_list_append (items, item_two);
+	items = g_list_append (items, item_three);
+
+	secret_item_load_secrets (items, NULL,
+	                          on_async_result, &result);
+	g_assert (result == NULL);
+
+	egg_test_wait ();
+
+	ret = secret_item_load_secrets_finish (result, &error);
+	g_assert_no_error (error);
+	g_object_unref (result);
+	g_assert (ret == TRUE);
+
+	value = secret_item_get_secret (item_one);
+	g_assert (value != NULL);
+	password = secret_value_get (value, &length);
+	g_assert_cmpuint (length, ==, 3);
+	g_assert_cmpstr (password, ==, "111");
+	secret_value_unref (value);
+
+	value = secret_item_get_secret (item_two);
+	g_assert (value != NULL);
+	password = secret_value_get (value, &length);
+	g_assert_cmpuint (length, ==, 3);
+	g_assert_cmpstr (password, ==, "222");
+	secret_value_unref (value);
+
+	value = secret_item_get_secret (item_three);
+	g_assert (value == NULL);
+
+	g_object_unref (item_one);
+	g_object_unref (item_two);
+	g_object_unref (item_three);
+	g_list_free (items);
+}
+
 static void
 test_delete_sync (Test *test,
                   gconstpointer unused)
@@ -610,7 +760,7 @@ test_delete_sync (Test *test,
 	SecretItem *item;
 	gboolean ret;
 
-	item = secret_item_new_sync (test->service, item_path, NULL, &error);
+	item = secret_item_new_for_dbus_path_sync (test->service, item_path, SECRET_ITEM_NONE, NULL, &error);
 	g_assert_no_error (error);
 
 	ret = secret_item_delete_sync (item, NULL, &error);
@@ -619,7 +769,7 @@ test_delete_sync (Test *test,
 
 	g_object_unref (item);
 
-	item = secret_item_new_sync (test->service, item_path, NULL, &error);
+	item = secret_item_new_for_dbus_path_sync (test->service, item_path, SECRET_ITEM_NONE, NULL, &error);
 	g_assert_error (error, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD);
 	g_assert (item == NULL);
 }
@@ -634,7 +784,7 @@ test_delete_async (Test *test,
 	SecretItem *item;
 	gboolean ret;
 
-	item = secret_item_new_sync (test->service, item_path, NULL, &error);
+	item = secret_item_new_for_dbus_path_sync (test->service, item_path, SECRET_ITEM_NONE, NULL, &error);
 	g_assert_no_error (error);
 
 	secret_item_delete (item, NULL, on_async_result, &result);
@@ -648,7 +798,7 @@ test_delete_async (Test *test,
 
 	g_object_unref (item);
 
-	item = secret_item_new_sync (test->service, item_path, NULL, &error);
+	item = secret_item_new_for_dbus_path_sync (test->service, item_path, SECRET_ITEM_NONE, NULL, &error);
 	g_assert_error (error, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD);
 	g_assert (item == NULL);
 }
@@ -673,9 +823,11 @@ main (int argc, char **argv)
 	g_test_add ("/item/set-attributes-sync", Test, "mock-service-normal.py", setup, test_set_attributes_sync, teardown);
 	g_test_add ("/item/set-attributes-async", Test, "mock-service-normal.py", setup, test_set_attributes_async, teardown);
 	g_test_add ("/item/set-attributes-prop", Test, "mock-service-normal.py", setup, test_set_attributes_prop, teardown);
-	g_test_add ("/item/get-secret-sync", Test, "mock-service-normal.py", setup, test_get_secret_sync, teardown);
-	g_test_add ("/item/get-secret-async", Test, "mock-service-normal.py", setup, test_get_secret_async, teardown);
+	g_test_add ("/item/load-secret-sync", Test, "mock-service-normal.py", setup, test_load_secret_sync, teardown);
+	g_test_add ("/item/load-secret-async", Test, "mock-service-normal.py", setup, test_load_secret_async, teardown);
 	g_test_add ("/item/set-secret-sync", Test, "mock-service-normal.py", setup, test_set_secret_sync, teardown);
+	g_test_add ("/item/secrets-sync", Test, "mock-service-normal.py", setup, test_secrets_sync, teardown);
+	g_test_add ("/item/secrets-async", Test, "mock-service-normal.py", setup, test_secrets_async, teardown);
 	g_test_add ("/item/delete-sync", Test, "mock-service-normal.py", setup, test_delete_sync, teardown);
 	g_test_add ("/item/delete-async", Test, "mock-service-normal.py", setup, test_delete_async, teardown);
 
